@@ -8,16 +8,19 @@ and clearing cached values based on those changes.
 
 import numpy as np
 
-import hashlib
 import zlib
+import time
+import hashlib
+
+from functools import wraps
 
 from .constants import log
 from .util import is_sequence
 
 try:
-    # xxhash is roughly 5x faster than adler32 but is only
+    # xxhash is roughly 5x faster than zlib.adler32 but is only
     # packaged in easy wheels on linux (`pip install xxhash`)
-    # so keep it as a soft dependency
+    # so we keep it a soft dependency
     import xxhash
     hasX = True
 except ImportError:
@@ -33,39 +36,98 @@ def tracked_array(array, dtype=None):
 
     Parameters
     ------------
-    array: array- like object to be turned into a TrackedArray
-    dtype: which dtype to use for the array
+    array : array- like object
+      To be turned into a TrackedArray
+    dtype : np.dtype
+      Which dtype to use for the array
 
     Returns
     ------------
-    tracked: TrackedArray, of input array data
+    tracked : TrackedArray
+      Contains input array data
     """
+    # make sure it is contiguous then view it as our subclass
     tracked = np.ascontiguousarray(array,
                                    dtype=dtype).view(TrackedArray)
+    # should always be contiguous here
     assert tracked.flags['C_CONTIGUOUS']
 
     return tracked
 
 
+def cache_decorator(function):
+    """
+    A decorator for class methods, replaces @property
+    but will store and retrieve function return values
+    in object cache.
+
+    Parameters
+    ------------
+    function : method
+      This is used as a decorator:
+      ```
+      @cache_decorator
+      def foo(self, things):
+        return 'happy days'
+      ```
+    """
+
+    # use wraps to preseve docstring
+    @wraps(function)
+    def get_cached(*args, **kwargs):
+        """
+        Only execute the function if its value isn't stored
+        in cache already.
+        """
+        self = args[0]
+        # use function name as key in cache
+        name = function.__name__
+        # do the dump logic ourselves to avoid
+        # verifying cache twice per call
+        self._cache.verify()
+        # access cache dict to avoid automatic verification
+        if name in self._cache.cache:
+            # already stored so return value
+            return self._cache.cache[name]
+
+        # time execution
+        tic = time.time()
+        # value not in cache so execute the function
+        value = function(*args, **kwargs)
+        # store the value
+        self._cache.cache[name] = value
+        # debug log execution time
+        # this is nice for debugging as you can see when
+        # cache is getting dumped all the time
+        log.debug('%s was not in cache, executed in %.6f',
+                  name,
+                  time.time() - tic)
+        return value
+
+    # all cached values are also properties
+    # so they can be accessed like value attributes
+    # rather than functions
+    return property(get_cached)
+
+
 class TrackedArray(np.ndarray):
     """
-    Track changes in a numpy ndarray.
+    Subclass of numpy.ndarray that provides hash methods
+    to track changes.
 
-    General method is to agressivly set 'modified' flags
-    on operations which might alter the array.
+    General method is to aggressively set 'modified' flags
+    on operations which might (but don't necessarily) alter
+    the array, ideally we sometimes compute hashes when we
+    don't need to, but we don't return wrong hashes ever.
 
     We store boolean modified flag for each hash type to
     make checks fast even for queries of different hashes.
 
-    This will force a recompute of a checksum value in some
-    cases where nothing has changed but we don't ever want to
-    return a checksum that doesn't reflect the data.
-
     Methods
     ----------
-    md5:       returns str, hexadecimal MD5 of array
-    crc:       returns int, zlib crc32/adler32 checksum
-    fast_hash: returns int, CRC or xxhash.xx64
+    md5 :       str, hexadecimal MD5 of array
+    crc :       int, zlib crc32/adler32 checksum
+    fast_hash : int, CRC or xxhash.xx64
     """
 
     def __array_finalize__(self, obj):
@@ -307,26 +369,6 @@ class Cache:
         self._lock = 0
         self.cache = {}
 
-    def get(self, key):
-        """
-        Get a key from the cache.
-
-        If the key is unavailable or the cache has been invalidated
-        returns None.
-
-        Parameters
-        -------------
-        key: hashable value
-
-        Returns
-        -------------
-        value: value from cache
-        """
-        self.verify()
-        if key in self.cache:
-            return self.cache[key]
-        return None
-
     def delete(self, key):
         """
         Remove a key from the cache.
@@ -340,14 +382,25 @@ class Cache:
         value of id_function and delete all stored items if
         the value of id_function has changed.
         """
+        # if we are in a lock don't check anything
+        if self._lock != 0:
+            return
+
+        # check the hash of our data
         id_new = self._id_function()
-        if (self._lock == 0) and (id_new != self.id_current):
+
+        # things changed
+        if id_new != self.id_current:
             if len(self.cache) > 0:
                 log.debug('%d items cleared from cache: %s',
                           len(self.cache),
                           str(list(self.cache.keys())))
-            self.clear()
-            self.id_set()
+            # hash changed, so dump the cache
+            # do it manually rather than calling clear()
+            # as we are internal logic and can avoid function calls
+            self.cache = {}
+            # set the id to the new data hash
+            self.id_current = id_new
 
     def clear(self, exclude=None):
         """
@@ -373,17 +426,40 @@ class Cache:
         """
         self.id_current = self._id_function()
 
-    def set(self, key, value):
+    def __getitem__(self, key):
+        """
+        Get an item from the cache. If the item
+        is not in the cache, it will return None
+
+        Parameters
+        -------------
+        key : hashable
+               Key in dict
+
+        Returns
+        -------------
+        cached : object, or None
+        """
+        self.verify()
+        if key in self.cache:
+            return self.cache[key]
+        return None
+
+    def __setitem__(self, key, value):
+        """
+        Add an item to the cache.
+
+        Parameters
+        ------------
+        key : hashable
+                 Key to reference value
+        value : any
+                  Value to store in cache
+        """
         self.verify()
         self.cache[key] = value
         return value
 
-    def __getitem__(self, key):
-        return self.get(key)
-
-    def __setitem__(self, key, value):
-        return self.set(key, value)
-
     def __contains__(self, key):
         self.verify()
         return key in self.cache
