@@ -9,24 +9,45 @@ from .base import Trimesh
 from .constants import log, tol
 from .triangles import normals
 from .geometry import faces_to_edges
-from .grouping import group_rows, unique_rows
 
 from . import util
+from . import grouping
 from . import transformations
 
 import numpy as np
-
-from collections import deque
+import collections
 
 try:
+    # shapely is a soft dependency
     from shapely.geometry import Polygon
     from shapely.wkb import loads as load_wkb
-except ImportError:
-    log.warning('shapely.geometry.Polygon not available!')
+except BaseException:
+    # shapely will sometimes raise OSErrors
+    # on import rather than just ImportError
+    log.warning('shapely.geometry.Polygon not available!',
+                exc_info=True)
 
 
 def validate_polygon(obj):
-    if util.is_instance_named(obj, 'Polygon'):
+    """
+    Make sure an input can be returned as a valid polygon.
+
+    Parameters
+    -------------
+    obj : shapely.geometry.Polygon, str (wkb), or (n, 2) float
+      Object which might be a polygon
+
+    Returns
+    ------------
+    polygon : shapely.geometry.Polygon
+      Valid polygon object
+
+    Raises
+    -------------
+    ValueError
+      If a valid finite- area polygon isn't available
+    """
+    if isinstance(obj, Polygon):
         polygon = obj
     elif util.is_shape(obj, (-1, 2)):
         polygon = Polygon(obj)
@@ -49,12 +70,17 @@ def extrude_polygon(polygon,
 
     Parameters
     ----------
-    polygon: shapely.geometry.Polygon object
-    height:  float, distance to extrude polygon along Z
+    polygon : shapely.geometry.Polygon
+      2D geometry to extrude
+    height : float
+      Distance to extrude polygon along Z
+    **kwargs:
+        passed to Trimesh
 
     Returns
     ----------
-    mesh: Trimesh object of result
+    mesh : trimesh.Trimesh
+      Resulting extrusion as watertight body
     """
     vertices, faces = triangulate_polygon(polygon, **kwargs)
     mesh = extrude_triangulation(vertices=vertices,
@@ -69,20 +95,26 @@ def sweep_polygon(polygon,
                   angles=None,
                   **kwargs):
     """
-    Extrude a 2D shapely polygon into a watertight 3D mesh
-    along an arbitrary 3D path.
+    Extrude a 2D shapely polygon into a 3D mesh along an
+    arbitrary 3D path. Doesn't handle sharp curvature.
+
 
     Parameters
     ----------
-    polygon: shapely.geometry.Polygon object
-    path:    (n,3) float, a path in 3D
-    angles:  (n,) float, optional rotation angle relative to prior vertex
-                         at each vertex
+    polygon : shapely.geometry.Polygon
+      Profile to sweep along path
+    path : (n, 3) float
+      A path in 3D
+    angles :  (n,) float
+      Optional rotation angle relative to prior vertex
+      at each vertex
 
     Returns
     -------
-    mesh : Trimesh object of result
+    mesh : trimesh.Trimesh
+      Geometry of result
     """
+
     path = np.asanyarray(path, dtype=np.float64)
     if not util.is_shape(path, (-1, 3)):
         raise ValueError('Path must be (n, 3)!')
@@ -101,8 +133,10 @@ def sweep_polygon(polygon,
     # Compute 3D locations of those vertices
     verts_3d = np.c_[verts_2d, np.zeros(n)]
     verts_3d = transformations.transform_points(verts_3d, tf_mat)
-    base_verts_3d = np.c_[base_verts_2d, np.zeros(len(base_verts_2d))]
-    base_verts_3d = transformations.transform_points(base_verts_3d, tf_mat)
+    base_verts_3d = np.c_[base_verts_2d,
+                          np.zeros(len(base_verts_2d))]
+    base_verts_3d = transformations.transform_points(base_verts_3d,
+                                                     tf_mat)
 
     # keep matching sequence of vertices and 0- indexed faces
     vertices = [base_verts_3d]
@@ -172,38 +206,45 @@ def extrude_triangulation(vertices,
                           height,
                           **kwargs):
     """
-    Turn a shapely.geometry Polygon object and a height (float)
-    into a watertight Trimesh object.
+    Turn a 2D triangulation into a watertight Trimesh.
 
     Parameters
     ----------
-    vertices: (n,2) float, 2D vertices
-    faces:    (m,3) int,   triangle indexes of vertices
-    height:   float, distance to extrude triangulation
+    vertices : (n, 2) float
+      2D vertices
+    faces : (m, 3) int
+      Triangle indexes of vertices
+    height : float
+      Distance to extrude triangulation
+    **kwargs:
+        passed to Trimesh
 
     Returns
     ---------
-    mesh: Trimesh object of result
+    mesh : trimesh.Trimesh
+      Mesh created from extrusion
     """
     vertices = np.asanyarray(vertices, dtype=np.float64)
-    faces = np.asanyarray(faces, dtype=np.int)
     height = float(height)
+    faces = np.asanyarray(faces, dtype=np.int64)
 
     if not util.is_shape(vertices, (-1, 2)):
-        raise ValueError('Vertices must be (n,3)')
+        raise ValueError('Vertices must be (n,2)')
     if not util.is_shape(faces, (-1, 3)):
         raise ValueError('Faces must be (n,3)')
-    if np.abs(height) < tol.zero:
+    if np.abs(height) < tol.merge:
         raise ValueError('Height must be nonzero!')
 
     # make sure triangulation winding is pointing up
     normal_test = normals(
         [util.three_dimensionalize(vertices[faces[0]])[1]])[0]
 
+    normal_dot = np.dot(normal_test,
+                        [0.0, 0.0, np.sign(height)])[0]
+
     # make sure the triangulation is aligned with the sign of
     # the height we've been passed
-    if np.dot(normal_test,
-              [0, 0, np.sign(height)]) < 0:
+    if normal_dot < 0.0:
         faces = np.fliplr(faces)
 
     # stack the (n,3) faces into (3*n, 2) edges
@@ -212,7 +253,7 @@ def extrude_triangulation(vertices,
     # edges which only occur once are on the boundary of the polygon
     # since the triangulation may have subdivided the boundary of the
     # shapely polygon, we need to find it again
-    edges_unique = group_rows(edges_sorted, require_count=1)
+    edges_unique = grouping.group_rows(edges_sorted, require_count=1)
 
     # (n, 2, 2) set of line segments (positions, not references)
     boundary = vertices[edges[edges_unique]]
@@ -242,18 +283,32 @@ def extrude_triangulation(vertices,
 
     mesh = Trimesh(*util.append_faces(vertices_seq,
                                       faces_seq),
-                   process=True)
+                   process=True,
+                   **kwargs)
+
+    assert mesh.volume > 0.0
+
     return mesh
 
 
-def triangulate_polygon(polygon, **kwargs):
+def triangulate_polygon(polygon,
+                        triangle_args='pq30',
+                        engine='auto',
+                        **kwargs):
     """
-    Given a shapely polygon create a triangulation using
-    meshpy.triangle
+    Given a shapely polygon create a triangulation using one
+    of the python interfaces to triangle.c:
+    > pip install meshpy
+    > pip install triangle
 
     Parameters
     ---------
-    polygon: Shapely.geometry.Polygon
+    polygon : Shapely.geometry.Polygon
+        Polygon object to be triangulated
+    triangle_args : str
+        Passed to triangle.triangulate
+    engine : str
+        'meshpy', 'triangle', or 'auto'
     kwargs: passed directly to meshpy.triangle.build:
             triangle.build(mesh_info,
                            verbose=False,
@@ -267,27 +322,46 @@ def triangulate_polygon(polygon, **kwargs):
                            generate_edges=None,
                            generate_faces=False,
                            min_angle=None)
+
     Returns
-    --------
-    mesh_vertices: (n, 2) float array of 2D points
-    mesh_faces:    (n, 3) int array of vertex indexes
+    --------------
+    vertices : (n, 2) float
+       Points in space
+    faces :    (n, 3) int
+       Index of vertices that make up triangles
     """
-    # do the import here, as sometimes this import can segfault
-    # which is not catchable with a try/except block
-    import meshpy.triangle as triangle
 
     # turn the polygon in to vertices, segments, and hole points
     arg = _polygon_to_kwargs(polygon)
-    # call meshpy.triangle on our cleaned representation of
-    # the Shapely polygon
+
+    try:
+        if str(engine).strip() in ['auto', 'triangle']:
+            from triangle import triangulate
+            result = triangulate(arg, triangle_args)
+            return result['vertices'], result['triangles']
+    except ImportError:
+        # no `triangle` so move on to `meshpy`
+        pass
+    except BaseException as E:
+        # if we see an exception log it and move on
+        log.error('failed to triangulate using triangle!',
+                  exc_info=True)
+        # if we are running unit tests exit here and fail
+        if tol.strict:
+            raise E
+
+    # do the import here, as sometimes this import can segfault
+    # which is not catchable with a try/except block
+    from meshpy import triangle
+    # call meshpy.triangle on our cleaned representation
     info = triangle.MeshInfo()
     info.set_points(arg['vertices'])
     info.set_facets(arg['segments'])
-    info.set_holes(arg['holes'])
-
+    # not all polygons have holes
+    if 'holes' in arg:
+        info.set_holes(arg['holes'])
     # build mesh and pass kwargs to triangle
     mesh = triangle.build(info, **kwargs)
-
     # (n, 2) float vertices
     vertices = np.array(mesh.points, dtype=np.float64)
     # (m, 3) int faces
@@ -303,11 +377,13 @@ def _polygon_to_kwargs(polygon):
 
     Parameters
     ---------
-    polygon: Shapely.geometry.Polygon
+    polygon : Shapely.geometry.Polygon
+      Input geometry
 
     Returns
     --------
-    result: dict, with keys: vertices, segments, holes
+    result : dict
+      Has keys: vertices, segments, holes
     """
 
     if not polygon.is_valid:
@@ -331,12 +407,12 @@ def _polygon_to_kwargs(polygon):
     def add_boundary(boundary, start):
         # coords is an (n, 2) ordered list of points on the polygon boundary
         # the first and last points are the same, and there are no
-        # guarentees on points not being duplicated (which will
+        # guarantees on points not being duplicated (which will
         # later cause meshpy/triangle to shit a brick)
         coords = np.array(boundary.coords)
         # find indices points which occur only once, and sort them
         # to maintain order
-        unique = np.sort(unique_rows(coords)[0])
+        unique = np.sort(grouping.unique_rows(coords)[0])
         cleaned = coords[unique]
 
         vertices.append(cleaned)
@@ -353,11 +429,11 @@ def _polygon_to_kwargs(polygon):
         return len(cleaned)
 
     # sequence of (n,2) points in space
-    vertices = deque()
+    vertices = collections.deque()
     # sequence of (n,2) indices of vertices
-    facets = deque()
+    facets = collections.deque()
     # list of (2) vertices in interior of hole regions
-    holes = deque()
+    holes = collections.deque()
 
     start = add_boundary(polygon.exterior, 0)
     for interior in polygon.interiors:
@@ -373,30 +449,40 @@ def _polygon_to_kwargs(polygon):
     vertices = np.vstack(vertices)
     facets = np.vstack(facets).tolist()
 
+    # shapely polygons can include a Z component
+    # strip it out for the triangulation
+    if vertices.shape[1] == 3:
+        vertices = vertices[:, :2]
+
+    result = {'vertices': vertices,
+              'segments': facets}
     # holes in meshpy lingo are a (h, 2) list of (x,y) points
     # which are inside the region of the hole
     # we added a hole for the exterior, which we slice away here
     holes = np.array(holes)[1:]
-
-    result = {'vertices': vertices,
-              'segments': facets,
-              'holes': holes}
+    if len(holes) > 0:
+        result['holes'] = holes
 
     return result
 
 
-def box(extents=None, transform=None):
+def box(extents=None, transform=None, **kwargs):
     """
     Return a cuboid.
 
     Parameters
     ------------
-    extents: float, or (3,) float edge length
-    transform: (4, 4) float, transformation matrix
+    extents : float, or (3,) float
+      Edge lengths
+    transform: (4, 4) float
+      Transformation matrix
+    **kwargs:
+        passed to Trimesh to create box
 
     Returns
     ------------
-    box: Trimesh object
+    box : trimesh.Trimesh
+      Cuboid geometry in space
     """
     vertices = [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
                 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1]
@@ -425,7 +511,8 @@ def box(extents=None, transform=None):
     box = Trimesh(vertices=vertices,
                   faces=faces,
                   face_normals=face_normals,
-                  process=False)
+                  process=False,
+                  **kwargs)
     if transform is not None:
         box.apply_transform(transform)
 
@@ -436,6 +523,10 @@ def icosahedron():
     """
     Create an icosahedron, a 20 faced polyhedron.
 
+    Returns
+    -------------
+    ico : trimesh.Trimesh
+      Icosahederon centered at the origin.
     """
     t = (1.0 + 5.0**.5) / 2.0
     vertices = [-1, t, 0, 1, t, 0, -1, -t, 0, 1, -t, 0, 0, -1, t, 0, 1, t,
@@ -444,8 +535,8 @@ def icosahedron():
              1, 5, 9, 5, 11, 4, 11, 10, 2, 10, 7, 6, 7, 1, 8,
              3, 9, 4, 3, 4, 2, 3, 2, 6, 3, 6, 8, 3, 8, 9,
              4, 9, 5, 2, 4, 11, 6, 2, 10, 8, 6, 7, 9, 8, 1]
-    # make every vertex have radius 1.0
-    vertices = np.reshape(vertices, (-1, 3)) / 1.9021130325903071
+    # scale vertices so each vertex radius is 1.0
+    vertices = np.reshape(vertices, (-1, 3)) / np.sqrt(2.0 + t)
     faces = np.reshape(faces, (-1, 3))
     mesh = Trimesh(vertices=vertices,
                    faces=faces,
@@ -459,14 +550,17 @@ def icosphere(subdivisions=3, radius=1.0):
 
     Parameters
     ----------
-    subdivisions: int, how many times to subdivide the mesh.
-                  Note that the number of faces will grow as function of
-                  4 ** subdivisions, so you probably want to keep this under ~5
-    radius: float, radius of resulting sphere
+    subdivisions : int
+      How many times to subdivide the mesh.
+      Note that the number of faces will grow as function of
+      4 ** subdivisions, so you probably want to keep this under ~5
+    radius : float
+      Desired radius of sphere
 
     Returns
     ---------
-    ico: trimesh.Trimesh object of sphere
+    ico : trimesh.Trimesh
+      Meshed sphere
     """
     def refine_spherical():
         vectors = ico.vertices
@@ -488,20 +582,25 @@ def uv_sphere(radius=1.0,
               theta=None,
               phi=None):
     """
-    Create a UV sphere (latitude + longitude) centered at the origin.
-
-    Roughly one order of magnitude faster than an icosphere but slightly uglier.
+    Create a UV sphere (latitude + longitude) centered at the
+    origin. Roughly one order of magnitude faster than an
+    icosphere but slightly uglier.
 
     Parameters
     ----------
-    radius: float, radius of sphere
-    count: (2,) int, number of lattitude and longitude lines
-    theta: (n,) float, optional
-    phi:   (n,) float, optional
+    radius : float
+      Radius of sphere
+    count : (2,) int
+      Number of latitude and longitude lines
+    theta : (n,) float
+      Optional theta angles in radians
+    phi :   (n,) float
+      Optional phi angles in radians
 
     Returns
     ----------
-    mesh: Trimesh object of UV sphere with specified parameters
+    mesh : trimesh.Trimesh
+       Mesh of UV sphere with specified parameters
     """
 
     count = np.array(count, dtype=np.int)
@@ -560,16 +659,20 @@ def capsule(height=1.0,
 
     Parameters
     ----------
-    height: float, center to center distance of two spheres
-    radius: float, radius of the cylinder and hemispheres
-    count:  (2,) int, number of sections on lattitude and longitude
+    height : float
+      Center to center distance of two spheres
+    radius : float
+      Radius of the cylinder and hemispheres
+    count : (2,) int
+      Number of sections on latitude and longitude
 
     Returns
     ----------
-    capsule: Trimesh of capsule with given properties
-             - cylinder axis is along Z
-             - one hemisphere is centered at the origin
-             - other hemisphere is centered along the Z axis at specified height
+    capsule : trimesh.Trimesh
+      Capsule geometry with:
+        - cylinder axis is along Z
+        - one hemisphere is centered at the origin
+        - other hemisphere is centered along the Z axis at height
     """
     height = float(height)
     radius = float(radius)
@@ -580,7 +683,8 @@ def capsule(height=1.0,
     # so that we can offset the top and bottom of a sphere to
     # get a nicely meshed capsule
     theta = np.linspace(0, np.pi, count[0])
-    center = np.clip(np.arctan(tol.merge / radius), tol.merge, np.inf)
+    center = np.clip(np.arctan(tol.merge / radius),
+                     tol.merge, np.inf)
     offset = np.array([-center, center]) + (np.pi / 2)
     theta = np.insert(theta,
                       int(len(theta) / 2),
@@ -596,19 +700,29 @@ def capsule(height=1.0,
     return capsule
 
 
-def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
+def cylinder(radius=1.0,
+             height=1.0,
+             sections=32,
+             transform=None,
+             **kwargs):
     """
     Create a mesh of a cylinder along Z centered at the origin.
 
     Parameters
     ----------
-    radius: float, the radius of the cylinder
-    height: float, the height of the cylinder
-    sections: int, how many pie wedges should the cylinder be meshed as
+    radius : float
+      The radius of the cylinder
+    height : float
+      The height of the cylinder
+    sections : int
+      How many pie wedges should the cylinder have
+    **kwargs:
+        passed to Trimesh to create cylinder
 
     Returns
     ----------
-    cylinder: Trimesh, resulting mesh
+    cylinder: trimesh.Trimesh
+      Resulting mesh of a cylinder
     """
 
     # create a 2D pie out of wedges
@@ -628,7 +742,8 @@ def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
     # extrude the 2D triangulation into a Trimesh object
     cylinder = extrude_triangulation(vertices=vertices,
                                      faces=faces,
-                                     height=height)
+                                     height=height,
+                                     **kwargs)
     # the extrusion was along +Z, so move the cylinder
     # center of mass back to the origin
     cylinder.vertices[:, 2] -= height * .5
@@ -640,19 +755,259 @@ def cylinder(radius=1.0, height=1.0, sections=32, transform=None):
     return cylinder
 
 
+def annulus(r_min=1.0,
+            r_max=2.0,
+            height=1.0,
+            sections=32,
+            transform=None,
+            **kwargs):
+    """
+    Create a mesh of an annular cylinder along Z,
+    centered at the origin.
+
+    Parameters
+    ----------
+    r_min : float
+      The inner radius of the annular cylinder
+    r_max : float
+      The outer radius of the annular cylinder
+    height : float
+      The height of the annular cylinder
+    sections : int
+      How many pie wedges should the annular cylinder have
+    **kwargs:
+        passed to Trimesh to create annulus
+
+    Returns
+    ----------
+    annulus : trimesh.Trimesh
+      Mesh of annular cylinder
+    """
+    r_min = abs(float(r_min))
+    r_max = abs(float(r_max))
+    height = float(height)
+    sections = int(sections)
+
+    # if center radius is zero this is a cylinder
+    if r_min < tol.merge:
+        return cylinder(radius=r_max,
+                        height=height,
+                        sections=sections,
+                        transform=transform)
+
+    # create a 2D pie out of wedges
+    theta = np.linspace(0, np.pi * 2, sections)[:-1]
+    unit = np.column_stack((np.sin(theta),
+                            np.cos(theta)))
+    assert len(unit) == sections - 1
+
+    vertices = np.vstack((unit * r_min,
+                          unit * r_max))
+
+    # one flattened triangulated quad covering one slice
+    face = np.array([0, sections - 1, 1,
+                     1, sections - 1, sections])
+
+    # tile one quad into lots of quads
+    faces = (np.tile(face, (sections - 1, 1)) +
+             np.arange(sections - 1).reshape((-1, 1))).reshape((-1, 3))
+
+    # stitch the last and first triangles with correct winding
+    faces[-1] = [sections - 1, 0, sections - 2]
+
+    # extrude the 2D profile into a mesh
+    annulus = extrude_triangulation(vertices=vertices,
+                                    faces=faces,
+                                    height=height,
+                                    **kwargs)
+
+    # move the annulus so the centroid is at the origin
+    annulus.vertices[:, 2] -= height * .5
+    if transform is not None:
+        # apply a transform here before any cache stuff is generated
+        # and would have to be dumped after the transform is applied
+        annulus.apply_transform(transform)
+
+    return annulus
+
+
 def random_soup(face_count=100):
     """
-    Return a random set of triangles as a Trimesh
+    Return random triangles as a Trimesh
 
     Parameters
     -----------
-    face_count: int, number of faces in resultant mesh
+    face_count : int
+      Number of faces desired in mesh
 
     Returns
     -----------
-    soup: Trimesh object with face_count random faces
+    soup : trimesh.Trimesh
+      Geometry with face_count random faces
     """
     vertices = np.random.random((face_count * 3, 3)) - 0.5
     faces = np.arange(face_count * 3).reshape((-1, 3))
     soup = Trimesh(vertices=vertices, faces=faces)
     return soup
+
+
+def axis(origin_size=0.04,
+         transform=None,
+         origin_color=None,
+         axis_radius=None,
+         axis_length=None):
+    """
+    Return an XYZ axis marker as a  Trimesh, which represents position
+    and orientation. If you set the origin size the other parameters
+    will be set relative to it.
+
+    Parameters
+    ----------
+    transform : (4, 4) float
+      Transformation matrix
+    origin_size : float
+      Radius of sphere that represents the origin
+    origin_color : (3,) float or int, uint8 or float
+      Color of the origin
+    axis_radius : float
+      Radius of cylinder that represents x, y, z axis
+    axis_length: float
+      Length of cylinder that represents x, y, z axis
+
+    Returns
+    -------
+    marker : trimesh.Trimesh
+      Mesh geometry of axis indicators
+    """
+    # the size of the ball representing the origin
+    origin_size = float(origin_size)
+
+    # set the transform and use origin-relative
+    # sized for other parameters if not specified
+    if transform is None:
+        transform = np.eye(4)
+    if origin_color is None:
+        origin_color = [255, 255, 255, 255]
+    if axis_radius is None:
+        axis_radius = origin_size / 5.0
+    if axis_length is None:
+        axis_length = origin_size * 10.0
+
+    # generate a ball for the origin
+    axis_origin = uv_sphere(radius=origin_size,
+                            count=[10, 10])
+    axis_origin.apply_transform(transform)
+
+    # apply color to the origin ball
+    axis_origin.visual.face_colors = origin_color
+
+    # create the cylinder for the z-axis
+    translation = transformations.translation_matrix(
+        [0, 0, axis_length / 2])
+    z_axis = cylinder(
+        radius=axis_radius,
+        height=axis_length,
+        transform=transform.dot(translation))
+    # XYZ->RGB, Z is blue
+    z_axis.visual.face_colors = [0, 0, 255]
+
+    # create the cylinder for the y-axis
+    translation = transformations.translation_matrix(
+        [0, 0, axis_length / 2])
+    rotation = transformations.rotation_matrix(np.radians(-90),
+                                               [1, 0, 0])
+    y_axis = cylinder(
+        radius=axis_radius,
+        height=axis_length,
+        transform=transform.dot(rotation).dot(translation))
+    # XYZ->RGB, Y is green
+    y_axis.visual.face_colors = [0, 255, 0]
+
+    # create the cylinder for the x-axis
+    translation = transformations.translation_matrix(
+        [0, 0, axis_length / 2])
+    rotation = transformations.rotation_matrix(np.radians(90),
+                                               [0, 1, 0])
+    x_axis = cylinder(
+        radius=axis_radius,
+        height=axis_length,
+        transform=transform.dot(rotation).dot(translation))
+    # XYZ->RGB, X is red
+    x_axis.visual.face_colors = [255, 0, 0]
+
+    # append the sphere and three cylinders
+    marker = util.concatenate([axis_origin,
+                               x_axis,
+                               y_axis,
+                               z_axis])
+    return marker
+
+
+def camera_marker(camera, marker_height=0.4, origin_size=None):
+    """
+    Create a visual marker for a camera object, including an axis and FOV.
+
+    Parameters
+    ---------------
+    camera : trimesh.scene.Camera
+      Camera object with FOV and transform defined
+    marker_height : float
+      How far along the camera Z should FOV indicators be
+    origin_size : float
+      Sphere radius of the origin (default: marker_height / 10.0)
+
+    Returns
+    ------------
+    meshes : list
+      Contains Trimesh and Path3D objects which can be visualized
+    """
+
+    # append the visualizations to an array
+    meshes = [axis(origin_size=marker_height / 10.0)]
+    meshes[0].apply_transform(camera.transform)
+
+    try:
+        # path is a soft dependency
+        from .path.exchange.load import load_path
+    except ImportError:
+        # they probably don't have shapely installed
+        log.warning('unable to create FOV visualization!',
+                    exc_info=True)
+        return meshes
+
+    # create sane origin size from marker height
+    if origin_size is None:
+        origin_size = marker_height / 10.0
+
+    # calculate vertices from camera FOV angles
+    x = marker_height * np.tan(np.deg2rad(camera.fov[0]) / 2.0)
+    y = marker_height * np.tan(np.deg2rad(camera.fov[1]) / 2.0)
+    z = marker_height
+
+    # combine the points into the vertices of an FOV visualization
+    points = [(0, 0, 0),
+              (-x, -y, z),
+              (x, -y, z),
+              (x, y, z),
+              (-x, y, z)]
+    # apply the camera extrinsic transform
+    points = transformations.transform_points(points, camera.transform)
+
+    # create line segments for the FOV visualization
+    # a segment from the origin to each bound of the FOV
+    segments = np.column_stack(
+        (np.zeros_like(points), points)).reshape(
+        (-1, 3))
+
+    # add a loop for the outside of the FOV then reshape
+    # the whole thing into multiple line segments
+    segments = np.vstack((segments,
+                          points[[1, 2,
+                                  2, 3,
+                                  3, 4,
+                                  4, 1]])).reshape((-1, 2, 3))
+
+    # add a single Path3D object for all line segments
+    meshes.append(load_path(segments))
+
+    return meshes
