@@ -1,9 +1,7 @@
 import numpy as np
 
-from .transformations import rotation_matrix
-from .constants import tol, log
-
 from . import util
+from .constants import tol, log
 
 try:
     from scipy.sparse import coo_matrix
@@ -13,74 +11,111 @@ except ImportError:
 
 def plane_transform(origin, normal):
     """
-    Given the origin and normal of a plane, find the transform that will move
-    that plane to be coplanar with the XY plane
+    Given the origin and normal of a plane find the transform
+    that will move that plane to be coplanar with the XY plane.
 
     Parameters
     ----------
-    origin: (3,) float, point in space
-    normal: (3,) float, plane normal vector
+    origin : (3,) float
+        Point that lies on the plane
+    normal : (3,) float
+        Vector that points along normal of plane
 
     Returns
     ---------
-    transform: (4,4) float, transformation matrix
+    transform: (4,4) float
+        Transformation matrix to move points onto XY plane
     """
     transform = align_vectors(normal, [0, 0, 1])
-    transform[0:3, 3] = -np.dot(transform, np.append(origin, 1))[0:3]
+    transform[0:3, 3] = -np.dot(transform,
+                                np.append(origin, 1))[0:3]
     return transform
 
 
-def align_vectors(vector_start, vector_end, return_angle=False):
+def align_vectors(a, b, return_angle=False):
     """
-    Returns the 4x4 transformation matrix which will rotate from
-    vector_start to vector_end, eg:
-
-    vector_end == np.dot(T, np.append(vector_start, 1))[0:3]
+    Find a transform between two 3D vectors.
 
+    Implements the method described here:
+    http://ethaneade.com/rot_between_vectors.pdf
 
     Parameters
-    -----------
-    vector_start: (3,) float, vector in space
-    vector_end:   (3,) float, vector in space
-    return_angle: bool, return angle between vectors or not
+    --------------
+    a : (3,) float
+      Source vector
+    b : (3,) float
+      Target vector
+    return_angle : bool
+      If True return the angle between the two vectors
 
     Returns
-    -----------
-    transform: (4,4) float, transformation matrix
-    angle:     float, angle in radians (only returned if flag set)
-
+    -------------
+    transform : (4, 4) float
+      Homogenous transform from a to b
+    angle : float
+      Angle between vectors in radians
+      Only returned if return_angle
     """
-    start = np.asanyarray(vector_start, dtype=np.float64)
-    start /= np.linalg.norm(start)
-    end = np.asanyarray(vector_end, dtype=np.float64)
-    end /= np.linalg.norm(end)
+    # copy of input vectors
+    a = np.array(a, dtype=np.float64, copy=True)
+    b = np.array(b, dtype=np.float64, copy=True)
 
-    cross = np.cross(start, end)
-    # we clip the norm to 1, as otherwise floating point bs
-    # can cause the arcsin to error
-    norm = np.linalg.norm(cross)
-    norm = np.clip(norm, -1.0, 1.0)
-    direction = np.sign(np.dot(start, end))
+    # make sure vectors are 3D
+    if a.shape != (3,) or b.shape != (3,):
+        raise ValueError('only works for (3,) vectors')
 
-    if norm < tol.zero:
-        # if the norm is zero, the vectors are the same
-        # and no rotation is needed
-        T = np.eye(4)
-        T[0:3] *= direction
+    # unitize input vectors
+    a /= np.linalg.norm(a)
+    b /= np.linalg.norm(b)
+
+    # projection of a onto b
+    dot = np.dot(a, b)
+
+    # are vectors just reversed
+    if dot < (tol.zero - 1):
+        # a reversed vector is 180 degrees
+        angle = np.pi
+
+        # get an arbitrary perpendicular vector to a
+        perp = util.generate_basis(a)[0] * np.eye(3)
+
+        # (3, 3) rotation from a to b
+        rotation = (2 * np.dot(perp, perp.T)) - np.eye(3)
+
+    # are vectors already the same
+    elif dot > (1 - tol.zero):
         angle = 0.0
-    else:
-        angle = np.arcsin(norm)
-        if direction < 0:
-            angle = np.pi - angle
-        T = rotation_matrix(angle, cross)
+        # no rotation
+        rotation = np.eye(3)
 
-    check = np.abs(np.dot(T[:3, :3], start) - end)
-    if not (check < 1e-5).all():
-        raise ValueError('aligning vectors failed!')
+    # vectors are at some angle to each other
+    else:
+        # we already handled values out of the range [-1.0, 1.0]
+        angle = np.arccos(dot)
+
+        # (3,) vector perpendicular to both a and b
+        w = np.cross(a, b)
+
+        # a float between 0.5 and 1.0
+        c = 1.0 / (1.0 + dot)
+
+        # (3, 3) skew- symmetric matrix from the (3,) vector w
+        # the matrix has the property: wx == -wx.T
+        wx = np.array([[0, -w[2], w[1]],
+                       [w[2], 0, -w[0]],
+                       [-w[1], w[0], 0]])
+
+        # (3, 3) rotation from a to b
+        rotation = np.eye(3) + wx + (np.dot(wx, wx) * c)
+
+    # put rotation into homogenous transformation matrix
+    transform = np.eye(4)
+    transform[:3, :3] = rotation
 
     if return_angle:
-        return T, angle
-    return T
+        return transform, angle
+
+    return transform
 
 
 def faces_to_edges(faces, return_index=False):
@@ -89,47 +124,54 @@ def faces_to_edges(faces, return_index=False):
 
     Parameters
     -----------
-    faces: (n,3) int, vertex indices representing faces
+    faces : (n, 3) int
+      Vertex indices representing faces
 
     Returns
     -----------
-    edges: (n*3, 2) int, vertex indices representing edges
+    edges : (n*3, 2) int
+      Vertex indices representing edges
     """
     faces = np.asanyarray(faces)
-    edges = np.column_stack((faces[:, (0, 1)],
-                             faces[:, (1, 2)],
-                             faces[:, (2, 0)])).reshape(-1, 2)
+
+    # each face has three edges
+    edges = faces[:, [0, 1, 1, 2, 2, 0]].reshape((-1, 2))
+
     if return_index:
-        face_index = np.tile(np.arange(len(faces)), (3, 1)).T.reshape(-1)
+        # edges are in order of faces due to reshape
+        face_index = np.tile(np.arange(len(faces)),
+                             (3, 1)).T.reshape(-1)
         return edges, face_index
     return edges
 
 
 def vector_angle(pairs):
     """
-    Find the angles between vector pairs
+    Find the angles between pairs of unit vectors.
 
     Parameters
     ----------
-    pairs: (n,2,3) set of unit vector pairs
+    pairs : (n,2,3) float
+      Unit vector pairs
 
     Returns
     ----------
-    angles: (n,) float, angles between vectors
-
-    Examples
-    ----------
-    angles = mesh.face_normals[mesh.face_adjacency]
+    angles : (n,) float
+      Angles between vectors in radians
     """
-    pairs = np.asanyarray(pairs)
+    pairs = np.asanyarray(pairs, dtype=np.float64)
     if len(pairs) == 0:
         return np.array([])
-    elif not util.is_shape(pairs, (-1, 2, 3)):
-        raise ValueError('pairs must be (n,2,3)!')
+    elif not util.is_shape(pairs, (-1, 2, (2, 3))):
+        raise ValueError('pairs must be (n,2,(2|3))!')
+
+    # do the dot product between vectors
     dots = util.diagonal_dot(pairs[:, 0], pairs[:, 1])
     # clip for floating point error
     dots = np.clip(dots, -1.0, 1.0)
+    # do cos and remove arbitrary sign
     angles = np.abs(np.arccos(dots))
+
     return angles
 
 
@@ -139,11 +181,13 @@ def triangulate_quads(quads):
 
     Parameters
     -----------
-    quads: (n,4) int, vertex indices of quad faces
+    quads: (n, 4) int
+      Vertex indices of quad faces
 
     Returns
     -----------
-    faces: (m,3) int, vertex indices of triangular faces
+    faces : (m, 3) int
+      Vertex indices of triangular faces
     """
     if len(quads) == 0:
         return quads
@@ -153,20 +197,27 @@ def triangulate_quads(quads):
     return faces
 
 
-def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
+def mean_vertex_normals(vertex_count,
+                        faces,
+                        face_normals,
+                        **kwargs):
     """
     Find vertex normals from the mean of the faces that contain that vertex.
 
     Parameters
     -----------
-    vertex_count: int, the number of vertices faces refer to
-    faces:        (n,3) int, list of vertex indices
-    face_normals: (n,3) float, normal vector for each face
+    vertex_count : int
+      The number of vertices faces refer to
+    faces : (n, 3) int
+      List of vertex indices
+    face_normals : (n, 3) float
+      Normal vector for each face
 
     Returns
     -----------
-    vertex_normals: (vertex_count, 3) float normals for every vertex
-                    Uncontained vertices will be zero.
+    vertex_normals : (vertex_count, 3) float
+      Normals for every vertex
+      Vertices unreferenced by faces will be zero.
     """
     def summed_sparse():
         # use a sparse matrix of which face contains each vertex to
@@ -177,7 +228,6 @@ def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
         else:
             sparse = index_sparse(vertex_count, faces)
         summed = sparse.dot(face_normals)
-        log.debug('Generated vertex normals using sparse matrix')
         return summed
 
     def summed_loop():
@@ -191,12 +241,13 @@ def mean_vertex_normals(vertex_count, faces, face_normals, **kwargs):
     try:
         summed = summed_sparse()
     except BaseException:
-        log.warning('Unable to generate sparse matrix! Falling back!',
-                    exc_info=True)
+        log.warning(
+            'unable to generate sparse matrix! Falling back!',
+            exc_info=True)
         summed = summed_loop()
-    unit_normals, valid = util.unitize(summed, check_valid=True)
-    vertex_normals = np.zeros((vertex_count, 3), dtype=np.float64)
-    vertex_normals[valid] = unit_normals
+
+    # invalid normals will be returned as zero
+    vertex_normals = util.unitize(summed)
 
     return vertex_normals
 
