@@ -42,41 +42,49 @@ def face_adjacency(faces=None,
 
     Parameters
     ----------
-    faces:        (n, d) int, set of faces referencing vertices by index
-    mesh:         Trimesh object, optional if passed will used cached edges
-    return_edges: bool, return the edges shared by adjacent faces
+    faces : (n, 3) int, or None
+        List of vertex indices representing triangles
+    mesh : Trimesh object
+        If passed will used cached edges instead of faces
+    return_edges : bool
+        Return the edges shared by adjacent faces
 
     Returns
     ---------
-    adjacency: (m,2) int, indexes of faces that are adjacent
-
-    if return_edges:
-         edges: (m,2) int, indexes of vertices which make up the
-                 edges shared by the adjacent faces
+    adjacency : (m,2) int
+        Indexes of faces that are adjacent
+    edges: (m,2) int
+        Only returned if return_edges is True
+        Indexes of vertices which make up the
+        edges shared by the adjacent faces
 
     Examples
     ----------
-    This is useful for lots of things, such as finding connected components:
-
-    graph = nx.Graph()
-    graph.add_edges_from(mesh.face_adjacency)
-    groups = nx.connected_components(graph_connected)
+    This is useful for lots of things such as finding
+    face- connected components:
+    >>> graph = nx.Graph()
+    >>> graph.add_edges_from(mesh.face_adjacency)
+    >>> groups = nx.connected_components(graph_connected)
     """
 
     if mesh is None:
         # first generate the list of edges for the current faces
         # also return the index for which face the edge is from
-        edges, edges_face = faces_to_edges(faces, return_index=True)
+        edges, edges_face = faces_to_edges(faces,
+                                           return_index=True)
+        # make sure edge rows are sorted
         edges.sort(axis=1)
     else:
-        # if passed a mesh, used the cached values for edges sorted
+        # if passed a mesh, used the cached values
         edges = mesh.edges_sorted
         edges_face = mesh.edges_face
 
     # this will return the indices for duplicate edges
     # every edge appears twice in a well constructed mesh
-    # so for every row in edge_idx, edges[edge_idx[*][0]] == edges[edge_idx[*][1]]
-    # in this call to group rows, we discard edges which don't occur twice
+    # so for every row in edge_idx:
+    # edges[edge_idx[*][0]] == edges[edge_idx[*][1]]
+    # in this call to group rows we discard edges which
+    # don't occur twice
     edge_groups = grouping.group_rows(edges, require_count=2)
 
     if len(edge_groups) == 0:
@@ -103,16 +111,18 @@ def face_adjacency_unshared(mesh):
 
     Parameters
     ----------
-    mesh: Trimesh object
+    mesh : Trimesh object
 
     Returns
     -----------
-    vid_unshared: (len(mesh.face_adjacency), 2) int, indexes of mesh.vertices
+    vid_unshared : (len(mesh.face_adjacency), 2) int
+        Indexes of mesh.vertices
     """
 
     # the non- shared vertex index is the same shape as face_adjacnecy
     # just holding vertex indices rather than face indices
-    vid_unshared = np.zeros_like(mesh.face_adjacency, dtype=np.int64)
+    vid_unshared = np.zeros_like(mesh.face_adjacency,
+                                 dtype=np.int64)
     # loop through both columns of face adjacency
     for i, adjacency in enumerate(mesh.face_adjacency.T):
         # faces from the current column of face adjacency
@@ -130,14 +140,16 @@ def face_adjacency_radius(mesh):
 
     Parameters
     --------------
-    mesh: Trimesh object
+    mesh : trimesh.Trimesh
 
     Returns
     -------------
-    radii: (n,) float, approximate radius between faces.
-           Parallel faces will have a value of np.inf
-    span:  (n,) float, perpendicular projection distance of two
-           unshared vertices onto the shared edge
+    radii : (len(self.face_adjacency),) float
+        Approximate radius between faces
+        Parallel faces will have a value of np.inf
+    span :  (len(self.face_adjacency),) float
+        Perpendicular projection distance of two
+        unshared vertices onto the shared edge
     """
 
     # solve for the radius of the adjacent faces
@@ -145,8 +157,8 @@ def face_adjacency_radius(mesh):
     # R = ------------------
     #     2 * sin(theta / 2)
     nonzero = mesh.face_adjacency_angles > np.radians(.01)
-    denominator = np.abs(2.0 *
-                         np.sin(mesh.face_adjacency_angles[nonzero] / 1.0))
+    denominator = np.abs(
+        2.0 * np.sin(mesh.face_adjacency_angles[nonzero] / 1.0))
 
     # consider the distance between the non- shared vertices of the
     # face adjacency pair as the key distance
@@ -177,30 +189,27 @@ def face_adjacency_radius(mesh):
 
 def vertex_adjacency_graph(mesh):
     """
-    Returns a networkx graph representing the vertices and their connections
-    in the mesh.
+    Returns a networkx graph representing the vertices and
+    their connections in the mesh.
 
     Parameters
     ----------
-    mesh:         Trimesh object
+    mesh : Trimesh object
 
     Returns
     ---------
-    graph: networkx.Graph(), graph representing vertices and edges between
-                             them,where vertices are networkx Nodes and edges
-                             are Edges.
+    graph : networkx.Graph
+        Graph representing vertices and edges between
+        them where vertices are nodes and edges are edges
 
     Examples
     ----------
     This is useful for getting nearby vertices for a given vertex,
     potentially for some simple smoothing techniques.
-
-
-    graph = mesh.vertex_adjacency_graph
-    graph.neighbors(0)
+    >>> graph = mesh.vertex_adjacency_graph
+    >>> graph.neighbors(0)
     > [1,3,4]
     """
-
     g = nx.Graph()
     g.add_edges_from(mesh.edges_unique)
     return g
@@ -246,13 +255,16 @@ def facets(mesh, engine=None):
 
     Parameters
     ---------
-    mesh:  Trimesh
-    engine: str, which graph engine to use ('scipy', 'networkx', 'graphtool')
+    mesh :  trimesh.Trimesh
+    engine : str
+       Which graph engine to use:
+       ('scipy', 'networkx', 'graphtool')
 
     Returns
     ---------
-    facets: list of groups of face indexes (mesh.faces) of parallel
-                  adjacent faces.
+    facets : sequence of (n,) int
+        Groups of face indexes of
+        parallel adjacent faces.
     """
     # what is the radius of a circle that passes through the perpendicular
     # projection of the vector between the two non- shared vertices
@@ -439,7 +451,7 @@ def connected_components(edges,
                                        ('scipy', components_csgraph),
                                        ('networkx', components_networkx)))
 
-    # if a graph engine has explictly been requested use it
+    # if a graph engine has explicitly been requested use it
     if engine in engines:
         return engines[engine]()
 
@@ -460,22 +472,160 @@ def connected_component_labels(edges, node_count=None):
 
     Parameters
     ----------
-    edges: (n, 2) int, edges of a graph
-    node_count: int, the largest node in the graph.
+    edges : (n, 2) int
+       Edges of a graph
+    node_count : int, or None
+        The largest node in the graph.
 
     Returns
     ---------
-    labels: (node_count,) int, component labels for each node
+    labels : (node_count,) int
+        Component labels for each node
     """
     matrix = edges_to_coo(edges, node_count)
-    body_count, labels = csgraph.connected_components(matrix,
-                                                      directed=False)
+    body_count, labels = csgraph.connected_components(
+        matrix, directed=False)
 
     assert len(labels) == node_count
 
     return labels
 
 
+def split_traversal(traversal,
+                    edges,
+                    edges_hash=None):
+    """
+    Given a traversal as a list of nodes, split the traversal
+    if a sequential index pair is not in the given edges.
+
+    Parameters
+    --------------
+    edges : (n, 2) int
+       Graph edge indexes
+    traversal : (m,) int
+       Traversal through edges
+    edge_hash : (n,)
+       Edges sorted on axis=1 and
+       passed to grouping.hashable_rows
+
+    Returns
+    ---------------
+    split : sequence of (p,) int
+    """
+    traversal = np.asanyarray(traversal,
+                              dtype=np.int64)
+
+    # hash edge rows for contains checks
+    if edges_hash is None:
+        edges_hash = grouping.hashable_rows(
+            np.sort(edges, axis=1))
+
+    # turn the (n,) traversal into (n-1,2) edges
+    trav_edge = np.column_stack((traversal[:-1],
+                                 traversal[1:]))
+    # hash each edge so we can compare to edge set
+    trav_hash = grouping.hashable_rows(
+        np.sort(trav_edge, axis=1))
+    # check if each edge is contained in edge set
+    contained = np.in1d(trav_hash, edges_hash)
+
+    # exit early if every edge of traversal exists
+    if contained.all():
+        # just reshape one traversal
+        split = [traversal]
+    else:
+        # find contiguous groups of contained edges
+        blocks = grouping.blocks(contained,
+                                 min_len=1,
+                                 only_nonzero=True)
+
+        # turn edges back in to sequence of traversals
+        split = [np.append(trav_edge[b][:, 0],
+                           trav_edge[b[-1]][1])
+                 for b in blocks]
+
+    # close traversals if necessary
+    for i, t in enumerate(split):
+        # make sure elements of sequence are numpy arrays
+        split[i] = np.asanyarray(split[i], dtype=np.int64)
+        # don't close if its a single edge
+        if len(t) <= 2:
+            continue
+        # make sure it's not already closed
+        edge = np.sort([t[0], t[-1]])
+        if edge.ptp() == 0:
+            continue
+        close = grouping.hashable_rows(edge.reshape((1, 2)))[0]
+        # if we need the edge add it
+        if close in edges_hash:
+            split[i] = np.append(t, t[0]).astype(np.int64)
+    result = np.array(split)
+
+    return result
+
+
+def fill_traversals(traversals, edges, edges_hash=None):
+    """
+    Convert a traversal of a list of edges into a sequence of
+    traversals where every pair of consecutive node indexes
+    is an edge in a passed edge list
+
+    Parameters
+    -------------
+    traversals : sequence of (m,) int
+       Node indexes of traversals of a graph
+    edges : (n, 2) int
+       Pairs of connected node indexes
+    edges_hash : None, or (n,) int
+       Edges sorted along axis 1 then hashed
+       using grouping.hashable_rows
+
+    Returns
+    --------------
+    splits : sequence of (p,) int
+       Node indexes of connected traversals
+    """
+    # make sure edges are correct type
+    edges = np.asanyarray(edges, dtype=np.int64)
+    # make sure edges are sorted
+    edges.sort(axis=1)
+
+    # if there are no traversals just return edges
+    if len(traversals) == 0:
+        return edges.copy()
+
+    # hash edges for contains checks
+    if edges_hash is None:
+        edges_hash = grouping.hashable_rows(edges)
+
+    splits = []
+    for nodes in traversals:
+        # split traversals to remove edges
+        # that don't actually exist
+        splits.extend(split_traversal(
+            traversal=nodes,
+            edges=edges,
+            edges_hash=edges_hash))
+    # turn the split traversals back into (n,2) edges
+    included = util.vstack_empty([np.column_stack((i[:-1], i[1:]))
+                                  for i in splits])
+    if len(included) > 0:
+        # sort included edges in place
+        included.sort(axis=1)
+        # make sure any edges not included in split traversals
+        # are just added as a length 2 traversal
+        splits.extend(grouping.boolean_rows(
+            edges,
+            included,
+            operation=np.setdiff1d))
+    else:
+        # no edges were included, so our filled traversal
+        # is just the original edges copied over
+        splits = edges.copy()
+
+    return splits
+
+
 def traversals(edges, mode='bfs'):
     """
     Given an edge list, generate a sequence of ordered
@@ -483,7 +633,8 @@ def traversals(edges, mode='bfs'):
 
     Parameters
     ------------
-    edges: (n,2) int, undirected edges of a graph
+    edges : (n,2) int, undirected edges of a graph
+    mode :  str, 'bfs', or 'dfs'
 
     Returns
     -----------
@@ -491,9 +642,12 @@ def traversals(edges, mode='bfs'):
                 ordered DFS or BFS traversals of the graph.
     """
     edges = np.asanyarray(edges, dtype=np.int64)
-    if not util.is_shape(edges, (-1, 2)):
+    if len(edges) == 0:
+        return []
+    elif not util.is_shape(edges, (-1, 2)):
         raise ValueError('edges are not (n,2)!')
 
+    # pick the traversal method
     mode = str(mode).lower().strip()
     if mode == 'bfs':
         func = csgraph.breadth_first_order
@@ -505,7 +659,6 @@ def traversals(edges, mode='bfs'):
     # make sure edges are sorted so we can query
     # an ordered pair later
     edges.sort(axis=1)
-
     # set of nodes to make sure we get every node
     nodes = set(edges.reshape(-1))
     # coo_matrix for csgraph routines
@@ -513,7 +666,6 @@ def traversals(edges, mode='bfs'):
 
     # we're going to make a sequence of traversals
     traversals = []
-    traversals_edge = []
 
     while len(nodes) > 0:
         # starting at any node
@@ -522,14 +674,8 @@ def traversals(edges, mode='bfs'):
         ordered = func(graph,
                        i_start=start,
                        return_predecessors=False,
-                       directed=False)
+                       directed=False).astype(np.int64)
 
-        # even if the traversal is closed there won't be an
-        # indication from the DFS, so add the first node
-        # to the end of the path
-        if np.sort(ordered[[0, -1]]) in edges:
-            ordered = np.append(ordered, ordered[0])
-        # add the traversal to our result
         traversals.append(ordered)
         # remove the nodes we've consumed
         nodes.difference_update(ordered)
@@ -537,61 +683,81 @@ def traversals(edges, mode='bfs'):
     return traversals
 
 
-def edges_to_coo(edges, count=None):
+def edges_to_coo(edges, count=None, data=None):
     """
     Given an edge list, return a boolean scipy.sparse.coo_matrix
     representing the edges in matrix form.
 
     Parameters
     ------------
-    edges: (n,2) int, edges of a graph
-    node_count: int, the number of nodes.
-                defaults to edges.max() + 1
+    edges : (n,2) int
+      Edges of a graph
+    count : int
+      The total number of nodes in the graph
+      if None: count = edges.max() + 1
+    data : (n,) any
+      Assign data to each edge, if None will
+      be bool True for each specified edge
 
     Returns
     ------------
-    matrix: (count, count) bool, scipy.sparse.coo_matrix
+    matrix: (count, count) scipy.sparse.coo_matrix
+      Sparse COO
     """
     edges = np.asanyarray(edges, dtype=np.int64)
     if not (len(edges) == 0 or
             util.is_shape(edges, (-1, 2))):
         raise ValueError('edges must be (n,2)!')
 
+    # if count isn't specified just set it to largest
+    # value referenced in edges
     if count is None:
         count = edges.max() + 1
-    else:
-        count = int(count)
+    count = int(count)
 
-    matrix = coo_matrix((np.ones(len(edges), dtype=np.bool),
-                         (edges[:, 0], edges[:, 1])),
-                        dtype=np.bool,
+    # if no data is specified set every specified edge
+    # to True
+    if data is None:
+        data = np.ones(len(edges), dtype=np.bool)
+
+    matrix = coo_matrix((data, edges.T),
+                        dtype=data.dtype,
                         shape=(count, count))
-
     return matrix
 
 
 def smoothed(mesh, angle):
     """
     Return a non- watertight version of the mesh which will
-    render nicely with smooth shading.
+    render nicely with smooth shading by disconnecting faces
+    at sharp angles to each other.
 
     Parameters
     ---------
-    mesh:  Trimesh object
-    angle: float, angle in radians, adjacent faces which have normals
-           below this angle will be smoothed.
+    mesh :  trimesh.Trimesh
+      Source geometry
+    angle : float
+      Angle in radians, adjacent faces which have normals
+      below this angle will be smoothed
 
     Returns
     ---------
-    smooth: Trimesh object
+    smooth : trimesh.Trimesh
+      Geometry with disconnected face patches
     """
+    # if the mesh has no adjacent faces return a copy
     if len(mesh.face_adjacency) == 0:
-        return mesh
+        return mesh.copy()
+
+    # face pairs below angle threshold
     angle_ok = mesh.face_adjacency_angles <= angle
+    # subset of face adjacency
     adjacency = mesh.face_adjacency[angle_ok]
+    # list of connected groups of faces
     components = connected_components(adjacency,
                                       min_len=1,
                                       nodes=np.arange(len(mesh.faces)))
+    # get a submesh as a single appended Trimesh
     smooth = mesh.submesh(components,
                           only_watertight=False,
                           append=True)
@@ -602,14 +768,18 @@ def is_watertight(edges, edges_sorted=None):
     """
     Parameters
     ---------
-    edges:        (n, 2) int, set of vertex indices
-    edges_sorted: (n, 2) int, vertex indices sorted on axis 1
+    edges : (n, 2) int
+      List of vertex indices
+    edges_sorted : (n, 2) int
+      Pass vertex indices sorted on axis 1 as a speedup
 
     Returns
     ---------
-    watertight: boolean, whether every edge is shared by an even
-                number of faces
-    winding:    boolean, whether every shared edge is reversed
+    watertight : boolean
+      Whether every edge is shared by an even
+      number of faces
+    winding : boolean
+      Whether every shared edge is reversed
     """
     # passing edges_sorted is a speedup only
     if edges_sorted is None:
@@ -617,13 +787,14 @@ def is_watertight(edges, edges_sorted=None):
 
     # group sorted edges
     groups = grouping.group_rows(edges_sorted, require_count=2)
-    watertight = (len(groups) * 2) == len(edges)
+    watertight = bool((len(groups) * 2) == len(edges))
 
     # are opposing edges reversed
     opposing = edges[groups].reshape((-1, 4))[:, 1:3].T
-    winding = np.equal(*opposing).all()
+    # wrap the weird numpy bool
+    winding = bool(np.equal(*opposing).all())
 
-    return bool(watertight), bool(winding)
+    return watertight, winding
 
 
 def graph_to_svg(graph):
@@ -713,8 +884,7 @@ def multigraph_paths(G, source, cutoff=None):
 
 def multigraph_collect(G, traversal, attrib=None):
     """
-    Given a MultiDiGraph traversal, collect attributes along that
-    path.
+    Given a MultiDiGraph traversal, collect attributes along it.
 
     Parameters
     -------------
