@@ -7,9 +7,6 @@ Functions for grouping values and rows.
 
 import numpy as np
 
-from collections import deque
-
-
 from . import util
 from .constants import log, tol
 
@@ -19,12 +16,25 @@ except ImportError:
     log.warning('Scipy unavailable')
 
 
-def merge_vertices_hash(mesh):
+def merge_vertices(mesh, distance=None):
     """
-    Removes duplicate vertices, based on integer hashes.
-    This is roughly 20x faster than querying a KD tree in a loop
+    Removes duplicate vertices, based on integer hashes of
+    each row.
+
+    Parameters
+    -------------
+    mesh : Trimesh object
+      Mesh to merge vertices on
+    distance : float or None
+      If not specified uses tol.merge
     """
-    unique, inverse = unique_rows(mesh.vertices)
+    if distance is not None:
+        digits = util.decimal_to_digits(distance)
+    else:
+        digits = None
+    # unique rows
+    unique, inverse = unique_rows(mesh.vertices,
+                                  digits=digits)
     mesh.update_vertices(unique, inverse)
 
 
@@ -90,7 +100,7 @@ def hashable_rows(data, digits=None):
     hashable:  (n) length array of custom data which can be sorted
                 or used as hash keys
     """
-    # if there is no data return immediatly
+    # if there is no data return immediately
     if len(data) == 0:
         return np.array([])
 
@@ -102,7 +112,7 @@ def hashable_rows(data, digits=None):
         return as_int
 
     # if array is 2D and smallish, we can try bitbanging
-    # this is signifigantly faster than the custom dtype
+    # this is significantly faster than the custom dtype
     if len(as_int.shape) == 2 and as_int.shape[1] <= 4:
         # time for some righteous bitbanging
         # can we pack the whole row into a single 64 bit integer
@@ -162,8 +172,10 @@ def float_to_int(data, digits=None, dtype=np.int32):
     data_max = np.abs(data).max() * 10**digits
     # ignore passed dtype if we have something large
     dtype = [np.int32, np.int64][int(data_max > 2**31)]
-    # round, multiply by large number, and convert
-    as_int = (np.round(data, digits) * (10**digits)).astype(dtype)
+    # multiply by requested power of ten
+    # then subtract small epsilon to avoid "go either way" rounding
+    # then do the rounding and convert to integer
+    as_int = np.round((data * 10 ** digits) - 1e-6).astype(dtype)
 
     return as_int
 
@@ -171,7 +183,7 @@ def float_to_int(data, digits=None, dtype=np.int32):
 def unique_ordered(data):
     """
     Returns the same as np.unique, but ordered as per the
-    first occurance of the unique value in data.
+    first occurrence of the unique value in data.
 
     Examples
     ---------
@@ -229,7 +241,7 @@ def unique_float(data,
     Identical to the numpy.unique command, except evaluates floating point
     numbers, using a specified number of digits.
 
-    If digits isn't specified, the libray default TOL_MERGE will be used.
+    If digits isn't specified, the library default TOL_MERGE will be used.
     """
     data = np.asanyarray(data)
     as_int = float_to_int(data, digits)
@@ -404,19 +416,21 @@ def boolean_rows(a, b, operation=np.intersect1d):
 
     Parameters
     ---------
-    a: (n,d) array
-    b: (m,d) array
-    operation: boolean set operation function:
-               set.intersection
-               set.difference
-               set.union
+    a: (n, d) int
+        Array with row vectors
+    b: (m, d) int
+        Array with row vectors
+    operation : function
+        Numpy boolean set operation function:
+          -np.intersect1d
+          -np.setdiff1d
 
     Returns
     --------
     shared: (p, d) array containing rows in both a and b
     """
-    a = np.asanyarray(a)
-    b = np.asanyarray(b)
+    a = np.asanyarray(a, dtype=np.int64)
+    b = np.asanyarray(b, dtype=np.int64)
 
     av = a.view([('', a.dtype)] * a.shape[1]).ravel()
     bv = b.view([('', b.dtype)] * b.shape[1]).ravel()
@@ -434,15 +448,20 @@ def group_vectors(vectors,
 
     Parameters
     -----------
-    vectors: (n,3) float, vectors in space
-    angle:   float, angle in radians to group by
-    include_negative: bool, if True consider for example
-                      [0,0,1] and [0,0,-1] the same
+    vectors : (n,3) float
+        Direction vector
+    angle : float
+        Group vectors closer than this angle in radians
+    include_negative : bool
+        If True consider the same:
+        [0,0,1] and [0,0,-1]
 
     Returns
     ------------
-    new_vectors: (m,3) float, vectors in space
-    groups:      (m,) sequence of indicies in source vectors
+    new_vectors : (m,3) float
+        Direction vector
+    groups : (m,) sequence of int
+        Indices of source vectors
     """
 
     vectors = np.asanyarray(vectors, dtype=np.float64)
@@ -464,13 +483,17 @@ def group_distance(values, distance):
 
     Parameters
     ---------
-    points:   (n, d) points (of dimension d)
-    distance: max distance between points in a cluster
+    points :   (n, d) float
+        Points of dimension d
+    distance : float
+        Max distance between points in a cluster
 
     Returns
     ----------
-    unique: (m, d), median value of group
-    groups: (m)     sequence of indexes
+    unique : (m, d) float
+        Median value of each group
+    groups : (m) sequence of int
+        Indexes of points that make up a group
 
     """
     values = np.asanyarray(values,
@@ -482,7 +505,7 @@ def group_distance(values, distance):
 
     # (n, d) set of values that are unique
     unique = []
-    # (n) sequence of indicies in values
+    # (n) sequence of indices in values
     groups = []
 
     for index, value in enumerate(values):
@@ -502,25 +525,33 @@ def clusters(points, radius):
 
     Parameters
     ---------
-    points: (n, d) points (of dimension d)
-    radius: max distance between points in a cluster
+    points : (n, d) float
+        Points of dimension d
+    radius : float
+        Max distance between points in a cluster
 
     Returns
     ----------
-    groups: (m) sequence of indices for points
+    groups : (m,) sequence of int
+        Indices of points in a cluster
 
     """
     from . import graph
     tree = KDTree(points)
 
     # some versions return pairs as a set of tuples
-    pairs = list(tree.query_pairs(radius))
+    pairs = tree.query_pairs(r=radius, output_type='ndarray')
+    # group connected components
     groups = graph.connected_components(pairs)
 
     return groups
 
 
-def blocks(data, min_len=2, max_len=np.inf, digits=None, only_nonzero=False):
+def blocks(data,
+           min_len=2,
+           max_len=np.inf,
+           digits=None,
+           only_nonzero=False):
     """
     Given an array, find the indices of contiguous blocks
     of equal values.
@@ -560,34 +591,29 @@ def blocks(data, min_len=2, max_len=np.inf, digits=None, only_nonzero=False):
     return blocks
 
 
-def merge_intervals(intervals):
+def group_min(groups, data):
     """
-    Given a list of intervals, merge overlapping ranges into a single list
-    of non- overlapping ranges
+    Given a list of groups, find the minimum element of data within each group
 
     Parameters
     -----------
-    intervals: (n,2) list of [start, end] values for ranges
+    groups : (n,) sequence of (q,) int
+        Indexes of each group corresponding to each element in data
+    data : (m,)
+        The data that groups indexes reference
 
     Returns
     -----------
-    merged: (m,2) list of [start, end] values of ranges with no overlaps
+    minimums : (n,)
+        Minimum value of data per group
+
     """
-    def merge_generator(intervals):
-        intervals.sort(axis=1)
-        intervals = intervals[np.lexsort(intervals.T[::-1])]
-        low, high = intervals[0]
-        for current in intervals[1:]:
-            if current[0] <= high:
-                high = max(high, current[1])
-            else:
-                yield [low, high]
-                low, high = current
-        yield [low, high]
-
-    intervals = np.asanyarray(intervals)
-    if not util.is_shape(intervals, (-1, 2)):
-        raise ValueError('Intervals must be (n,2)!')
-
-    merged = np.array([i for i in merge_generator(intervals)])
-    return merged
+    # sort with major key groups, minor key data
+    order = np.lexsort((data, groups))
+    groups = groups[order]  # this is only needed if groups is unsorted
+    data = data[order]
+    # construct an index which marks borders between groups
+    index = np.empty(len(groups), 'bool')
+    index[0] = True
+    index[1:] = groups[1:] != groups[:-1]
+    return data[index]
