@@ -28,13 +28,13 @@ from .. import grouping
 from .. import transformations
 
 from . import raster
+from . import repair
 from . import simplify
-from . import entities
 from . import polygons
-from . import creation
+from . import segments
 from . import traversal
 
-from .io.export import export_path
+from .exchange.export import export_path
 
 try:
     # try running shapely speedups
@@ -66,13 +66,17 @@ class Path(object):
 
         Parameters
         -----------
-        entities: list of Entity objects
-        vertices: (n, dimension) float, vertices referenced by entities
-        metadata: dict, any metadata about the path
-        process:  bool, if True run simple cleanup operations
+        entities : (m,) trimesh.path.entities.Entity
+          Contains geometric entities
+        vertices : (n, dimension) float
+          The vertices referenced by entities
+        metadata : dict
+          Any metadata about the path
+        process :  bool
+          Run simple cleanup or not
         """
 
-        self.entities = np.array(entities)
+        self.entities = entities
         self.vertices = vertices
         self.metadata = dict()
 
@@ -82,8 +86,8 @@ class Path(object):
         self._cache = caching.Cache(id_function=self.crc)
 
         if process:
-            # literally nothing will work if vertices aren't
-            # merged properly
+            # literally nothing will work if vertices
+            # aren't merged properly
             self.merge_vertices()
 
     def process(self):
@@ -104,6 +108,14 @@ class Path(object):
     def vertices(self, values):
         self._vertices = caching.tracked_array(values)
 
+    @property
+    def entities(self):
+        return self._entities
+
+    @entities.setter
+    def entities(self, values):
+        self._entities = np.asanyarray(values)
+
     @property
     def layers(self):
         """
@@ -113,7 +125,7 @@ class Path(object):
         ---------
         layers: (len(entities), ) list of str
         """
-        layer = [None] * len(self.entities)
+        layer = ['NONE'] * len(self.entities)
         for i, e in enumerate(self.entities):
             if hasattr(e, 'layer'):
                 layer[i] = str(e.layer)
@@ -128,10 +140,10 @@ class Path(object):
         crc: int, CRC of entity points and vertices
         """
         # first CRC the points in every entity
-        target = caching.crc32(bytes().join(e.points.tostring()
+        target = caching.crc32(bytes().join(e._bytes()
                                             for e in self.entities))
         # add the CRC for the vertices
-        target += self.vertices.crc()
+        target ^= self.vertices.crc()
         return target
 
     def md5(self):
@@ -142,13 +154,15 @@ class Path(object):
         ------------
         md5: str, two appended MD5 hashes
         """
+        # first MD5 the points in every entity
+        target = '{}{}'.format(
+            util.md5_object(bytes().join(e._bytes()
+                                         for e in self.entities)),
+            self.vertices.md5())
 
-        target = util.md5_object(bytes().join(e.points.tostring()
-                                              for e in self.entities))
-        target += self.vertices.md5()
         return target
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def paths(self):
         """
         Sequence of closed paths, encoded by entity index.
@@ -157,10 +171,11 @@ class Path(object):
         ---------
         paths: (n,) sequence of (*,) int referencing self.entities
         """
-        paths = traversal.closed_paths(self.entities, self.vertices)
+        paths = traversal.closed_paths(self.entities,
+                                       self.vertices)
         return paths
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def dangling(self):
         """
         List of entities that aren't included in a closed path
@@ -177,7 +192,7 @@ class Path(object):
                                 included)
         return dangling
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def kdtree(self):
         """
         A KDTree object holding the vertices of the path.
@@ -198,13 +213,14 @@ class Path(object):
 
         Returns
         ----------
-        scale: float, approximate size of the world holding this path
+        scale : float
+            Approximate size of the world holding this path
         """
         # use vertices peak-peak rather than exact extents
         scale = float((self.vertices.ptp(axis=0) ** 2).sum() ** .5)
         return scale
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def bounds(self):
         """
         Return the axis aligned bounding box of the current path.
@@ -279,31 +295,17 @@ class Path(object):
             new_entities.extend(entity.explode())
         self.entities = np.array(new_entities)
 
-    def fill_gaps(self, max_distance=np.inf):
+    def fill_gaps(self, distance=0.025):
         """
-        Find vertexes with degree 1 and try to connect them to other
-        vertices of degree 1, in place.
+        Find vertices without degree 2 and try to connect to
+        other vertices. Operations are done in-place.
 
         Parameters
         ----------
-        max_distance: float, connect vertices up to this distance.
-                      Default is infinity, but something like path.scale/100
-                      may make more sense.
+        distance : float
+          Connect vertices up to this distance
         """
-
-        broken = np.array(
-            [k for k, v in dict(self.vertex_graph.degree()).items() if v == 1])
-        if len(broken) < 2:
-            return
-
-        distance, node = KDTree(self.vertices[broken]).query(
-            self.vertices[broken], k=2)
-        edges = broken[node]
-        ok = np.logical_and(distance[:, 1] < max_distance, [
-                            not self.vertex_graph.has_edge(*i) for i in edges])
-
-        self.entities = np.append(self.entities,
-                                  [entities.Line(i) for i in edges[ok]])
+        repair.fill_gaps(self, distance=distance)
 
     @property
     def is_closed(self):
@@ -314,10 +316,12 @@ class Path(object):
         -----------
         closed: every entity is connected at its ends
         """
-        closed = all(i == 2 for i in dict(self.vertex_graph.degree()).values())
+        closed = all(i == 2 for i in
+                     dict(self.vertex_graph.degree()).values())
+
         return closed
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def vertex_graph(self):
         """
         Return a networkx.Graph object for the entity connectiviy
@@ -327,6 +331,21 @@ class Path(object):
         graph, closed = traversal.vertex_graph(self.entities)
         return graph
 
+    @caching.cache_decorator
+    def vertex_nodes(self):
+        """
+        Get a list of which vertex indices are nodes,
+        which are either endpoints or points where the
+        entity makes a direction change.
+
+        Returns
+        --------------
+        nodes : (n, 2) int
+          Indexes of self.vertices which are nodes
+        """
+        nodes = np.vstack([e.nodes for e in self.entities])
+        return nodes
+
     def apply_transform(self, transform):
         """
         Apply a transformation matrix to the current path in- place
@@ -351,9 +370,10 @@ class Path(object):
         cache = {}
         # apply transform to discretized paths
         if 'discrete' in self._cache.cache:
-            cache['discrete'] = np.array([transformations.transform_points(d,
-                                                                           matrix=transform)
-                                          for d in self.discrete])
+            cache['discrete'] = np.array([
+                transformations.transform_points(
+                    d, matrix=transform)
+                for d in self.discrete])
 
         # things we can just straight up copy
         # as they are topological not geometric
@@ -370,8 +390,9 @@ class Path(object):
                 cache[key] = self._cache.cache[key]
 
         # transform vertices in place
-        self.vertices = transformations.transform_points(self.vertices,
-                                                         matrix=transform)
+        self.vertices = transformations.transform_points(
+            self.vertices,
+            matrix=transform)
         # explicitly clear the cache
         self._cache.clear()
         self._cache.id_set()
@@ -393,6 +414,27 @@ class Path(object):
         matrix[:dimension, :dimension] *= float(scale)
         self.apply_transform(matrix)
 
+    def apply_translation(self, offset):
+        """
+        Apply a transformation matrix to the current path in- place
+
+        Parameters
+        -----------
+        transform: (dimension + 1, dimension + 1) float, homogenous
+                   transformation matrix
+        """
+        # work on 2D and 3D paths
+        dimension = self.vertices.shape[1]
+        # make sure offset is correct length and type
+        offset = np.array(
+            offset, dtype=np.float64).reshape(dimension)
+        # create a homogenous transform
+        matrix = np.eye(dimension + 1)
+        # apply the offset
+        matrix[:dimension, dimension] = offset
+
+        self.apply_transform(matrix)
+
     def apply_layer(self, name):
         """
         Apply a layer name to every entity in the path.
@@ -421,24 +463,57 @@ class Path(object):
         self.apply_transform(matrix)
         return matrix
 
-    def merge_vertices(self):
+    def merge_vertices(self, digits=None):
         """
         Merges vertices which are identical and replace references.
 
+        Parameters
+        --------------
+        digits : None, or int
+          How many digits to consider when merging vertices
         Alters
         -----------
-        self.entities: entity.points re- referenced
-        self.vertices: duplicates removed
+        self.entities : entity.points re- referenced
+        self.vertices : duplicates removed
         """
-        digits = util.decimal_to_digits(tol.merge * self.scale,
-                                        min_digits=1)
+        if digits is None:
+            digits = util.decimal_to_digits(tol.merge * self.scale,
+                                            min_digits=1)
+
         unique, inverse = grouping.unique_rows(self.vertices,
                                                digits=digits)
         self.vertices = self.vertices[unique]
-        for entity in self.entities:
-            # if we merged duplicate vertices, the entity may contain
-            # multiple references to the same vertex
-            entity.points = grouping.merge_runs(inverse[entity.points])
+
+        entities_ok = np.ones(len(self.entities), dtype=np.bool)
+
+        for index, entity in enumerate(self.entities):
+            # what kind of entity are we dealing with
+            kind = type(entity).__name__
+
+            # entities that don't need runs merged
+            # don't screw up control- point- knot relationship
+            if kind in 'BSpline Bezier Text':
+                entity.points = inverse[entity.points]
+                continue
+            # if we merged duplicate vertices, the entity may
+            # have multiple references to the same vertex
+            points = grouping.merge_runs(inverse[entity.points])
+            # if there are three points and two are identical fix it
+            if kind == 'Line':
+                if len(points) == 3 and points[0] == points[-1]:
+                    points = points[:2]
+                elif len(points) < 2:
+                    # lines need two or more vertices
+                    entities_ok[index] = False
+            elif kind == 'Arc' and len(points) != 3:
+                # three point arcs need three points
+                entities_ok[index] = False
+
+            # store points in entity
+            entity.points = points
+
+        # remove degenerate entities
+        self.entities = self.entities[entities_ok]
 
     def replace_vertex_references(self, mask):
         """
@@ -446,11 +521,13 @@ class Path(object):
 
         Parameters
         ------------
-        mask: (len(self.vertices), ) int, contains new vertex indexes
+        mask : (len(self.vertices), ) int
+          Contains new vertex indexes
 
         Alters
         ------------
-        entity.points in self.entities: replaced by mask[entity.points]
+        entity.points in self.entities
+          Replaced by mask[entity.points]
         """
         for entity in self.entities:
             entity.points = mask[entity.points]
@@ -476,7 +553,8 @@ class Path(object):
         ----------
         self.entities: shortened
         """
-        valid = np.array([i.is_valid for i in self.entities], dtype=np.bool)
+        valid = np.array([i.is_valid for i in self.entities],
+                         dtype=np.bool)
         self.entities = self.entities[valid]
 
     def remove_duplicate_entities(self):
@@ -492,7 +570,7 @@ class Path(object):
         if len(unique) != len(self.entities):
             self.entities = np.array(self.entities)[unique]
 
-    @property
+    @caching.cache_decorator
     def referenced_vertices(self):
         """
         Which vertices are referenced by an entity.
@@ -542,7 +620,7 @@ class Path(object):
                                              scale=self.scale)
         return discrete
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def discrete(self):
         """
         A sequence of connected vertices in space, corresponding to
@@ -550,15 +628,36 @@ class Path(object):
 
         Returns
         ---------
-        discrete: (len(self.paths),) sequence of (m*, dimension) float
+        discrete : (len(self.paths),)
+            A sequence of (m*, dimension) float
         """
-        discrete = np.array([self.discretize_path(i) for i in self.paths])
+        discrete = np.array([self.discretize_path(i)
+                             for i in self.paths])
         return discrete
 
-    def export(self, file_obj=None, file_type='dict'):
+    def export(self,
+               file_obj=None,
+               file_type=None,
+               **kwargs):
+        """
+        Export the path to a file object or return data.
+
+        Parameters
+        ---------------
+        file_obj : None, str, or file object
+          File object or string to export to
+        file_type : None or str
+          Type of file: dxf, dict, svg
+
+        Returns
+        ---------------
+        exported : bytes or str
+          Exported as specified type
+        """
         return export_path(self,
                            file_type=file_type,
-                           file_obj=file_obj)
+                           file_obj=file_obj,
+                           **kwargs)
 
     def to_dict(self):
         export_dict = self.export(file_type='dict')
@@ -572,10 +671,21 @@ class Path(object):
         ---------
         copied: Path object, copy of self
         """
+
+        metadata = {}
+        # grab all the keys into a list so if something is added
+        # in another thread it probably doesn't stomp on our loop
+        for key in list(self.metadata.keys()):
+            try:
+                metadata[key] = copy.deepcopy(self.metadata[key])
+            except RuntimeError:
+                # multiple threads
+                log.warning('key {} changed during copy'.format(key))
+
         # copy the core data
         copied = type(self)(entities=copy.deepcopy(self.entities),
                             vertices=copy.deepcopy(self.vertices),
-                            metadata=copy.deepcopy(self.metadata))
+                            metadata=metadata)
 
         cache = {}
         # try to copy the cache over to the new object
@@ -596,11 +706,17 @@ class Path(object):
 
         return copied
 
-    def show(self):
-        if self.is_closed:
-            self.plot_discrete(show=True)
-        else:
-            self.plot_entities(show=True)
+    def scene(self):
+        """
+        Get a scene object containing the current Path3D object.
+
+        Returns
+        --------
+        scene: trimesh.scene.Scene object containing current path
+        """
+        from ..scene import Scene
+        scene = Scene(self)
+        return scene
 
     def __add__(self, other):
         """
@@ -628,7 +744,10 @@ class Path3D(Path):
                 self.generate_closed_paths,
                 self.generate_discrete]
 
-    def to_planar(self, to_2D=None, normal=None, check=True):
+    def to_planar(self,
+                  to_2D=None,
+                  normal=None,
+                  check=True):
         """
         Check to see if current vectors are all coplanar.
 
@@ -637,63 +756,108 @@ class Path3D(Path):
 
         Parameters
         -----------
-        to_2D: (4,4) float, transformation matrix to apply.
-                     If not passed a plane will be fitted to vertices.
-        normal: (3,) float, normal of plane which is only used if to_2D
-                      is not specified
-        check:  bool, raise a ValueError if the points aren't coplanar after
-                      being transformed
+        to_2D: (4,4) float
+            Homogenous transformation matrix to apply,
+            If not passed a plane will be fitted to vertices.
+        normal: (3,) float, or None
+           Approximate normal of direction of plane
+           If to_2D is not specified sign
+           will be applied to fit plane normal
+        check:  bool
+            If True: Raise a ValueError if
+            points aren't coplanar
 
         Returns
         -----------
-        planar: Path2D object, current path transformed onto a plane
-        to_3D:  (4,4), transformation matrix to move planar back into 3D space
+        planar : trimesh.path.Path2D
+                   Current path transformed onto plane
+        to_3D :  (4,4) float
+                   Homeogenous transformation to move planar
+                   back into 3D space
         """
+        # which vertices are actually referenced
+        referenced = self.referenced_vertices
+
+        # no explicit transform passed
         if to_2D is None:
-            C, N = plane_fit(self.vertices)
+            # fit a plane to our vertices
+            C, N = plane_fit(self.vertices[referenced])
+            # apply the normal sign hint
             if normal is not None:
-                N *= np.sign(np.dot(N, normal))
-                N = normal
-            to_2D = plane_transform(C, N)
+                normal = np.asanyarray(normal, dtype=np.float64)
+                if normal.shape == (3,):
+                    N *= np.sign(np.dot(N, normal))
+                    N = normal
+                else:
+                    log.warning(
+                        "passed normal not used: {}".format(
+                            normal.shape))
+            # create a transform from fit plane to XY
+            to_2D = plane_transform(origin=C,
+                                    normal=N)
 
-        flat = transformations.transform_points(self.vertices, to_2D)
+        # make sure we've extracted a transform
+        to_2D = np.asanyarray(to_2D, dtype=np.float64)
+        if to_2D.shape != (4, 4):
+            raise ValueError('unable to create transform!')
 
-        if check and np.any(np.std(flat[:, 2]) > tol.planar):
-            log.error('points have z with deviation %f', np.std(flat[:, 2]))
-            raise NameError('Points aren\'t planar!')
+        # transform all vertices to 2D plane
+        flat = transformations.transform_points(self.vertices,
+                                                to_2D)
 
-        planar = Path2D(entities=copy.deepcopy(self.entities),
-                        vertices=flat[:, 0:2],
-                        metadata=copy.deepcopy(self.metadata))
+        # Z values of vertices which are referenced
+        heights = flat[referenced][:, 2]
+        # points are not on a plane because Z varies
+        if heights.ptp() > tol.planar:
+            # since Z is inconsistent set height to zero
+            height = 0.0
+            if check:
+                raise ValueError('points are not flat!')
+        else:
+            # if the points were planar store the height
+            height = heights.mean()
+
+        # the transform from 2D to 3D
         to_3D = np.linalg.inv(to_2D)
 
+        # if the transform didn't move the path to
+        # exactly Z=0 adjust it so the returned transform does
+        if np.abs(height) > tol.planar:
+            # adjust to_3D transform by height
+            adjust = transformations.translation_matrix(
+                [0, 0, height])
+            # apply the height adjustment to_3D
+            to_3D = np.dot(to_3D, adjust)
+
+        # copy metadata to new object
+        metadata = copy.deepcopy(self.metadata)
+        # store transform we used to move it onto the plane
+        metadata['to_3D'] = to_3D
+
+        # create the Path2D with the same entities
+        # and XY values of vertices projected onto the plane
+        planar = Path2D(entities=copy.deepcopy(self.entities),
+                        vertices=flat[:, :2],
+                        metadata=metadata,
+                        process=False)
+
         return planar, to_3D
 
-    def scene(self):
-        """
-        Get a scene object containing the current Path3D object.
-
-        Returns
-        --------
-        scene: trimesh.scene.Scene object containing current path
-        """
-        from ..scene import Scene
-        scene = Scene(self)
-        return scene
-
-    def show(self):
+    def show(self, **kwargs):
         """
         Show the current Path3D object.
         """
-        self.scene().show()
+        scene = self.scene()
+        return scene.show(**kwargs)
 
     def plot_discrete(self, show=False):
         """
-        Plot the discrete closed curves.
+        Plot closed curves
 
         Parameters
         ------------
-        show:
+        show : bool
+           If False will not execute matplotlib.pyplot.show
         """
         import matplotlib.pyplot as plt
         from mpl_toolkits.mplot3d import Axes3D
@@ -705,6 +869,15 @@ class Path3D(Path):
             plt.show()
 
     def plot_entities(self, show=False):
+        """
+        Plot discrete version of entities without regards
+        for connectivity.
+
+        Parameters
+        -------------
+        show : bool
+           If False will not execute matplotlib.pyplot.show
+        """
         import matplotlib.pyplot as plt
         from mpl_toolkits.mplot3d import Axes3D
         fig = plt.figure()
@@ -718,7 +891,19 @@ class Path3D(Path):
 
 class Path2D(Path):
 
+    def show(self, annotations=True):
+        """
+        Plot the current Path2D object using matplotlib.
+        """
+        if self.is_closed:
+            self.plot_discrete(show=True, annotations=annotations)
+        else:
+            self.plot_entities(show=True, annotations=annotations)
+
     def _process_functions(self):
+        """
+        Return a list of functions to clean up a Path2D
+        """
         return [self.merge_vertices,
                 self.remove_duplicate_entities,
                 self.remove_unreferenced_vertices]
@@ -766,26 +951,79 @@ class Path2D(Path):
                                  width=width)
         return image
 
+    def sample(self, count, **kwargs):
+        """
+        Use rejection sampling to generate random points inside a
+        polygon.
+
+        Parameters
+        -----------
+        count   : int
+                    Number of points to return
+                    If there are multiple bodies, there will
+                    be up to count * bodies points returned
+        factor  : float
+                    How many points to test per loop
+                    IE, count * factor
+        max_iter : int,
+                    Maximum number of intersection loops
+                    to run, total points sampled is
+                    count * factor * max_iter
+
+        Returns
+        -----------
+        hit : (n, 2) float
+               Random points inside polygon
+        """
+
+        poly = self.polygons_full
+        if len(poly) == 0:
+            samples = np.array([])
+        elif len(poly) == 1:
+            samples = polygons.sample(poly[0], count=count, **kwargs)
+        else:
+            samples = util.vstack_empty([
+                polygons.sample(i, count=count, **kwargs)
+                for i in poly])
+
+        return samples
+
     @property
     def body_count(self):
         return len(self.root)
 
-    def to_3D(self):
+    def to_3D(self, transform=None):
         """
         Convert 2D path to 3D path on the XY plane.
 
+        Parameters
+        -------------
+        transform : (4, 4) float
+            If passed, will transform vertices.
+            If not passed and 'to_3D' is in metadata
+            that transform will be used.
+
         Returns
         -----------
         path_3D: Path3D version of current path
         """
-        vertices_new = np.column_stack((copy.deepcopy(self.vertices),
-                                        np.zeros(len(self.vertices))))
+        # if there is a stored 'to_3D' transform in metadata use it
+        if transform is None and 'to_3D' in self.metadata:
+            transform = self.metadata['to_3D']
+
+        # copy vertices and stack with zeros from (n, 2) to (n, 3)
+        vertices = np.column_stack((copy.deepcopy(self.vertices),
+                                    np.zeros(len(self.vertices))))
+        if transform is not None:
+            vertices = transformations.transform_points(vertices,
+                                                        transform)
+        # make sure everything is deep copied
         path_3D = Path3D(entities=copy.deepcopy(self.entities),
-                         vertices=vertices_new,
+                         vertices=vertices,
                          metadata=copy.deepcopy(self.metadata))
         return path_3D
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def polygons_closed(self):
         """
         Cycles in the vertex graph, as shapely.geometry.Polygons.
@@ -798,11 +1036,11 @@ class Path2D(Path):
         polygons_closed: (n,) list of shapely.geometry.Polygon objects
         """
         # will attempt to recover invalid garbage geometry
-        polys, valid = polygons.paths_to_polygons(self.discrete)
-        self._cache.set('path_valid', valid)
+        # and will be None if geometry is unrecoverable
+        polys = polygons.paths_to_polygons(self.discrete)
         return polys
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def polygons_full(self):
         """
         A list of shapely.geometry.Polygon objects with interiors created
@@ -810,24 +1048,36 @@ class Path2D(Path):
 
         Returns
         ---------
-        full: list of shapely.geometry.Polygon objects
+        full : (len(self.root),) shapely.geometry.Polygon
+            Polygons containing interiors
         """
-        full = [None] * len(self.enclosure_shell)
-        for i, (shell_index,
-                holes_index) in enumerate(self.enclosure_shell.items()):
-            # a list of multiple Polygon objects
-            holes_poly = self.polygons_closed[holes_index]
+        # pre- allocate the list to avoid indexing problems
+        full = [None] * len(self.root)
+        # store the graph to avoid cache thrashing
+        enclosure = self.enclosure_directed
+        # store closed polygons to avoid cache hits
+        closed = self.polygons_closed
+
+        # loop through root curves
+        for i, root in enumerate(self.root):
+            # a list of multiple Polygon objects that
+            # are fully contained by the root curve
+            children = [closed[child]
+                        for child in enclosure[root].keys()]
             # all polygons_closed are CCW, so for interiors reverse them
-            holes = [np.array(p.exterior.coords)[::-1] for p in holes_poly]
+            holes = [np.array(p.exterior.coords)[::-1]
+                     for p in children]
             # a single Polygon object
-            shell = self.polygons_closed[shell_index].exterior
+            shell = closed[root].exterior
             # create a polygon with interiors
             full[i] = polygons.repair_invalid(Polygon(shell=shell,
                                                       holes=holes))
+        # so we can use advanced indexing
+        full = np.array(full)
 
         return full
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def area(self):
         """
         Return the area of the polygons interior.
@@ -836,10 +1086,10 @@ class Path2D(Path):
         ---------
         area: float, total area of polygons minus interiors
         """
-        area = sum(i.area for i in self.polygons_full)
+        area = float(sum(i.area for i in self.polygons_full))
         return area
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def length(self):
         """
         The total discretized length of every entity.
@@ -882,55 +1132,85 @@ class Path2D(Path):
             return result[0]
         return result
 
+    def triangulate(self, **kwargs):
+        """
+        Create a region- aware triangulation of the 2D path.
+
+        Parameters
+        -------------
+        **kwargs : dict
+          Passed to trimesh.creation.triangulate_polygon
+
+        Returns
+        -------------
+        vertices : (n, 2) float
+          2D vertices of triangulation
+        faces : (n, 3) int
+          Indexes of vertices for triangles
+        """
+        from ..creation import triangulate_polygon
+
+        # append vertices and faces into sequence
+        v_seq = []
+        f_seq = []
+
+        # loop through polygons with interiors
+        for polygon in self.polygons_full:
+            v, f = triangulate_polygon(polygon, **kwargs)
+            v_seq.append(v)
+            f_seq.append(f)
+
+        return util.append_faces(v_seq, f_seq)
+
     def medial_axis(self, resolution=None, clip=None):
         """
         Find the approximate medial axis based
-        on a voronoi diagram of evenly spaced points on the boundary of the polygon.
+        on a voronoi diagram of evenly spaced points on the
+        boundary of the polygon.
 
         Parameters
         ----------
-        resolution: target distance between each sample on the polygon boundary
-        clip:       [minimum number of samples, maximum number of samples]
-                    specifying a very fine resolution can cause the sample count to
-                    explode, so clip specifies a minimum and maximum number of samples
-                    to use per boundary region. To not clip, this can be specified as:
-                    [0, np.inf]
+        resolution : None or float
+          Distance between each sample on the polygon boundary
+        clip : None, or (2,) float
+          Min, max number of samples
 
         Returns
         ----------
-        medial:     Path2D object
+        medial : Path2D object
+          Contains only medial axis of Path
         """
-        if 'medial' in self._cache:
-            return self._cache.get('medial')
-
         if resolution is None:
             resolution = self.scale / 1000.0
 
         medials = [polygons.medial_axis(i, resolution, clip)
                    for i in self.polygons_full]
         medials = np.sum(medials)
-        return self._cache.set(key='medial',
-                               value=medials)
+        return medials
 
     def connected_paths(self, path_id, include_self=False):
         """
-        Given an index of self.paths, find other paths which overlap with
-        that path.
+        Given an index of self.paths find other paths which
+        overlap with that path.
 
         Parameters
         -----------
-        path_id:      int, index of self.paths
-        include_self: bool, should the result include path_id or not
+        path_id : int
+          Index of self.paths
+        include_self : bool
+          Should the result include path_id or not
 
         Returns
         -----------
-        path_ids: (n,) int, indexes of self.paths that overlap input path_id
+        path_ids :  (n, ) int
+          Indexes of self.paths that overlap input path_id
         """
         if len(self.root) == 1:
             path_ids = np.arange(len(self.polygons_closed))
         else:
-            path_ids = list(nx.node_connected_component(self.enclosure,
-                                                        path_id))
+            path_ids = list(nx.node_connected_component(
+                self.enclosure,
+                path_id))
         if include_self:
             return np.array(path_ids)
         return np.setdiff1d(path_ids, [path_id])
@@ -942,7 +1222,7 @@ class Path2D(Path):
 
         Returns
         ---------
-        simplified: Path2D object
+        simplified : Path2D object
         """
         return simplify.simplify_basic(self, **kwargs)
 
@@ -952,8 +1232,10 @@ class Path2D(Path):
 
         Parameters
         -----------
-        path_indexes: (n) int list of indexes for self.paths
-        smooth:       float, how much the spline should smooth the curve
+        path_indexes : (n) int
+          List of indexes of self.paths to convert
+        smooth : float
+          How much the spline should smooth the curve
 
         Returns
         ------------
@@ -970,45 +1252,12 @@ class Path2D(Path):
 
         Returns
         ----------
-        split: (n,) list of Path2D objects
+        split:  (n,) list of Path2D objects
+          Each connected region and interiors
         """
-        if self.root is None or len(self.root) == 0:
-            split = []
-        else:
-            split = [None] * len(self.root)
-            for i, root in enumerate(self.root):
-                connected = self.connected_paths(root, include_self=True)
+        return traversal.split(self)
 
-                new_root = np.nonzero(connected == root)[0]
-                new_entities = collections.deque()
-                new_paths = collections.deque()
-                new_metadata = copy.deepcopy(self.metadata)
-                new_metadata['split_2D'] = i
-
-                for path in self.paths[self.path_valid][connected]:
-                    new_paths.append(np.arange(len(path)) + len(new_entities))
-                    new_entities.extend(path)
-                new_entities = np.array(new_entities)
-
-                assert len(new_paths) == len(connected)
-
-                # prevents the copying from nuking our cache
-                with self._cache:
-                    split[i] = Path2D(
-                        entities=copy.deepcopy(
-                            self.entities[new_entities]), vertices=copy.deepcopy(
-                            self.vertices), metadata=new_metadata)
-                    split[i]._cache.update(
-                        {'paths': np.array(new_paths),
-                         'polygons_closed': self.polygons_closed[connected],
-                         'path_valid': np.ones(len(new_paths), dtype=np.bool),
-                         'discrete': self.discrete[self.path_valid][connected],
-                         'root': new_root})
-        [i._cache.id_set() for i in split]
-        self._cache.id_set()
-        return np.array(split)
-
-    def plot_discrete(self, show=False):
+    def plot_discrete(self, show=False, annotations=True):
         """
         Plot the closed curves of the path.
         """
@@ -1019,11 +1268,18 @@ class Path2D(Path):
         for i, points in enumerate(self.discrete):
             color = ['g', 'k'][i in self.root]
             axis.plot(*points.T, color=color)
+
+        if annotations:
+            for e in self.entities:
+                if not hasattr(e, 'plot'):
+                    continue
+                e.plot(self.vertices)
+
         if show:
             plt.show()
         return axis
 
-    def plot_entities(self, show=False):
+    def plot_entities(self, show=False, annotations=True, color=None):
         """
         Plot the entities of the path, with no notion of topology
         """
@@ -1037,10 +1293,17 @@ class Path2D(Path):
                    'BSpline0': {'color': 'm', 'linewidth': 1},
                    'BSpline1': {'color': 'm', 'linewidth': 1}}
         for entity in self.entities:
+            if annotations and hasattr(entity, 'plot'):
+                entity.plot(self.vertices)
+                continue
             discrete = entity.discrete(self.vertices)
             e_key = entity.__class__.__name__ + str(int(entity.closed))
             fmt = eformat[e_key]
-            if hasattr(entity, 'color'):
+            if color is not None:
+                # passed color will override other optons
+                fmt['color'] = color
+            elif hasattr(entity, 'color'):
+                # if entity has specified color use it
                 fmt['color'] = entity.color
             plt.plot(*discrete.T, **fmt)
         if show:
@@ -1059,12 +1322,14 @@ class Path2D(Path):
             raise TypeError('Identifier only valid for single body')
         return polygons.polygon_hash(self.polygons_full[0])
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def identifier_md5(self):
         """
         Return an MD5 of the identifier
         """
-        return util.md5_array(self.identifier, digits=4)
+        as_int = (self.identifier * 1e4).astype(np.int64)
+        hashed = util.md5_object(as_int.tostring(order='C'))
+        return hashed
 
     @property
     def path_valid(self):
@@ -1074,10 +1339,11 @@ class Path2D(Path):
         path_valid: (n,) bool, indexes of self.paths self.polygons_closed
                          which are valid polygons
         """
-        exists = self.polygons_closed
-        return self._cache.get('path_valid')
+        valid = [i is not None for i in self.polygons_closed]
+        valid = np.array(valid, dtype=np.bool)
+        return valid
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def root(self):
         """
         Which indexes of self.paths/self.polygons_closed are root curves.
@@ -1090,7 +1356,7 @@ class Path2D(Path):
         populate = self.enclosure_directed
         return self._cache['root']
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def enclosure(self):
         """
         Networkx Graph object of polygon enclosure.
@@ -1099,16 +1365,16 @@ class Path2D(Path):
             undirected = self.enclosure_directed.to_undirected()
         return undirected
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def enclosure_directed(self):
         """
         Networkx DiGraph of polygon enclosure
         """
         root, enclosure = polygons.enclosure_tree(self.polygons_closed)
-        self._cache.set('root', root)
+        self._cache['root'] = root
         return enclosure
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def enclosure_shell(self):
         """
         A dictionary of path indexes which are 'shell' paths, and values
