@@ -1,7 +1,9 @@
 import numpy as np
 import networkx as nx
 
-from shapely.geometry import Polygon, Point, LineString
+from shapely.geometry import Polygon, Point
+from shapely import vectorized
+
 from rtree import Rtree
 from collections import deque
 
@@ -18,43 +20,81 @@ from .traversal import resample_path
 
 def enclosure_tree(polygons):
     """
-    Given a list of shapely polygons, which are the root (aka outermost)
-    polygons, and which represent the holes which penetrate the root
-    curve. We do this by creating an R-tree for rough collision detection,
-    and then do polygon queries for a final result
+    Given a list of shapely polygons with only exteriors,
+    find which curves represent the exterior shell or root curve
+    and which represent holes which penetrate the exterior.
+
+    This is done with an R-tree for rough overlap detection,
+    and then exact polygon queries for a final result.
 
     Parameters
     -----------
-    polygons: (n,) list of shapely.geometry.Polygon objects
+    polygons : (n,) shapely.geometry.Polygon
+       Polygons which only have exteriors and may overlap
 
     Returns
     -----------
-    roots: (m,) int, index of polygons which are root
-    contains:  networkx.DiGraph, edges indicate a polygon
-               contained by another polygon
+    roots : (m,) int
+        Index of polygons which are root
+    contains : networkx.DiGraph
+       Edges indicate a polygon is
+       contained by another polygon
     """
     tree = Rtree()
-    for i, polygon in enumerate(polygons):
-        tree.insert(i, polygon.bounds)
-    count = len(polygons)
     # nodes are indexes in polygons
     contains = nx.DiGraph()
-    # make sure every polygon is included
-    contains.add_nodes_from(np.arange(count))
-    for i in range(count):
-        if polygons[i] is None:
+    for i, polygon in enumerate(polygons):
+        # if a polygon is None it means creation
+        # failed due to weird geometry so ignore it
+        if polygon is None:
+            continue
+        # insert polygon bounds into rtree
+        tree.insert(i, polygon.bounds)
+        # make sure every valid polygon has a node
+        contains.add_node(i)
+
+    # loop through every polygon
+    for i, polygon in enumerate(polygons):
+        # if polygon creation failed ignore it
+        if polygon is None:
             continue
         # we first query for bounding box intersections from the R-tree
-        for j in tree.intersection(polygons[i].bounds):
+        for j in tree.intersection(polygon.bounds):
+            # if we are checking a polygon against itself continue
             if (i == j):
                 continue
-            # we then do a more accurate polygon in polygon test to generate
-            # the enclosure tree information
+            # do a more accurate polygon in polygon test
+            # for the enclosure tree information
             if polygons[i].contains(polygons[j]):
                 contains.add_edge(i, j)
             elif polygons[j].contains(polygons[i]):
                 contains.add_edge(j, i)
-    roots = [n for n, deg in dict(contains.in_degree()).items() if deg == 0]
+
+    # a root or exterior curve has an even number of parents
+    # wrap in dict call to avoid networkx view
+    degree = dict(contains.in_degree())
+
+    # convert keys and values to numpy arrays
+    indexes = np.array(list(degree.keys()))
+    degrees = np.array(list(degree.values()))
+
+    # roots are curves with an even inward degree (parent count)
+    roots = indexes[(degrees % 2) == 0]
+
+    # if there are multiple nested polygons split the graph
+    # so the contains logic returns the individual polygons
+    if len(degrees) > 0 and degrees.max() > 1:
+        # collect new edges for graph
+        edges = []
+        # find edges of subgraph for each root and children
+        for root in roots:
+            children = indexes[degrees == degree[root] + 1]
+            edges.extend(contains.subgraph(np.append(children, root)).edges())
+        # stack edges into new directed graph
+        contains = nx.from_edgelist(edges, nx.DiGraph())
+        # if roots have no children add them anyway
+        contains.add_nodes_from(roots)
+
     return roots, contains
 
 
@@ -138,34 +178,60 @@ def polygon_obb(polygon):
     return bounds.oriented_bounds_2D(points)
 
 
-def transform_polygon(polygon, transform, plot=False):
+def transform_polygon(polygon, matrix):
+    """
+    Transform a polygon by a a 2D homogenous transform.
+
+    Parameters
+    -------------
+    polygon : shapely.geometry.Polygon
+                 2D polygon to be transformed.
+    matrix  : (3, 3) float
+                 2D homogenous transformation.
+
+    Returns
+    --------------
+    result : shapely.geometry.Polygon
+                 Polygon transformed by matrix.
+
+    """
+    matrix = np.asanyarray(matrix, dtype=np.float64)
+
     if util.is_sequence(polygon):
-        result = [transform_polygon(p, t) for p, t in zip(polygon, transform)]
-    else:
-        shell = transform_points(np.array(polygon.exterior.coords), transform)
-        holes = [transform_points(np.array(i.coords), transform)
-                 for i in polygon.interiors]
-        result = Polygon(shell=shell, holes=holes)
-    if plot:
-        plot_polygon(result)
+        result = [transform_polygon(p, t)
+                  for p, t in zip(polygon, matrix)]
+        return result
+    # transform the outer shell
+    shell = transform_points(np.array(polygon.exterior.coords),
+                             matrix)[:, :2]
+    # transform the interiors
+    holes = [transform_points(np.array(i.coords),
+                              matrix)[:, :2]
+             for i in polygon.interiors]
+    # create a new polygon with the result
+    result = Polygon(shell=shell, holes=holes)
     return result
 
 
-def plot_polygon(polygon, show=True):
+def plot_polygon(polygon, show=True, **kwargs):
     """
     Plot a shapely polygon using matplotlib.
 
     Parameters
     ------------
-    polygon: shapely.geometry.Polygon object
-    show:    bool, if True will display immediatly
+    polygon : shapely.geometry.Polygon
+      Polygon to be plotted
+    show : bool
+      If True will display immediately
+    **kwargs
+      Passed to plt.plot
     """
     import matplotlib.pyplot as plt
 
     def plot_single(single):
-        plt.plot(*single.exterior.xy, color='b')
+        plt.plot(*single.exterior.xy, **kwargs)
         for interior in single.interiors:
-            plt.plot(*interior.xy, color='r')
+            plt.plot(*interior.xy, **kwargs)
     # make aspect ratio non- stupid
     plt.axes().set_aspect('equal', 'datalim')
     if util.is_sequence(polygon):
@@ -230,53 +296,59 @@ def stack_boundaries(boundaries):
     return result
 
 
-def medial_axis(polygon, resolution=.01, clip=None):
+def medial_axis(polygon,
+                resolution=None,
+                clip=None):
     """
-    Given a shapely polygon, find the approximate medial axis based
-    on a voronoi diagram of evenly spaced points on the boundary of the polygon.
+    Given a shapely polygon, find the approximate medial axis
+    using a voronoi diagram of evenly spaced points on the
+    boundary of the polygon.
 
     Parameters
     ----------
-    polygon:    a shapely.geometry.Polygon
-    resolution: target distance between each sample on the polygon boundary
-    clip:       [minimum number of samples, maximum number of samples]
-                specifying a very fine resolution can cause the sample count to
-                explode, so clip specifies a minimum and maximum number of samples
-                to use per boundary region. To not clip, this can be specified as:
-                [0, np.inf]
+    polygon : shapely.geometry.Polygon
+        The source geometry
+    resolution : float
+        Distance between each sample on the polygon boundary
+    clip : None, or (2,) float
+        Clip the lower and upper bound of sample count to:
+        [minimum number of samples, maximum number of samples]
+        specifying a very fine resolution can cause the sample count to
+        explode, so clip specifies a minimum and maximum number of samples
+        to use per boundary region. To not clip, this can be specified as:
+        [0, np.inf]
 
     Returns
     ----------
-    lines:     (n,2,2) set of line segments
+    medial : Path2D object
     """
     from scipy.spatial import Voronoi
-    from .entities import Line
     from .path import Path2D
+    from .exchange.misc import edges_to_path
 
+    if resolution is None:
+        resolution = .01
+
+    # get evenly spaced points on the polygons boundaries
     samples = resample_boundaries(polygon=polygon,
                                   resolution=resolution,
                                   clip=clip)
+    # stack the boundary into a (m,2) float array
     samples = stack_boundaries(samples)
-
-    # create the voronoi diagram, after vertically stacking the points
-    # deque from a sequnce into a clean (m,2) array
+    # create the voronoi diagram on 2D points
     voronoi = Voronoi(samples)
-    # which voronoi vertices are contained inside the original polygon
-    contain = np.array([polygon.contains(Point(i))
-                        for i in voronoi.vertices],
-                       dtype=np.bool)
+    # which voronoi vertices are contained inside the polygon
+    contains = vectorized.contains(polygon, *voronoi.vertices.T)
     # ridge vertices of -1 are outside, make sure they are False
-    contain = np.append(contain, False)
-    # is a ridge fully inside the polygon
-    inside = [i for i in voronoi.ridge_vertices if contain[i].all()]
-    edges = np.vstack([util.stack_lines(i)
-                       for i in inside if len(i) >= 2])
+    contains = np.append(contains, False)
+    # make sure ridge vertices is numpy array
+    ridge = np.asanyarray(voronoi.ridge_vertices, dtype=np.int64)
+    # only take ridges where every vertex is contained
+    edges = ridge[contains[ridge].all(axis=1)]
     # line objects from edges
-    entities = [Line(points=i) for i in edges]
-
-    # create the Path2D object for the medial axis
-    medial = Path2D(entities=entities,
-                    vertices=voronoi.vertices)
+    medial = Path2D(**edges_to_path(
+        edges=edges,
+        vertices=voronoi.vertices))
 
     return medial
 
@@ -342,21 +414,27 @@ class InversePolygon:
 
 def polygon_hash(polygon):
     """
-    An approximate hash of a a shapely Polygon object.
+    Return a vector containing values representitive of
+    a particular polygon.
 
     Parameters
     ---------
-    polygon: shapely.geometry.Polygon object
+    polygon : shapely.geometry.Polygon
+      Input geometry
 
     Returns
     ---------
-    hash: (5) length list of hash representing input polygon
+    hashed: (6), float
+      Representitive values representing input polygon
     """
-    result = [len(polygon.interiors),
-              polygon.convex_hull.area,
-              polygon.convex_hull.length,
-              polygon.area,
-              polygon.length]
+    result = np.array(
+        [len(polygon.interiors),
+         polygon.convex_hull.area,
+         polygon.convex_hull.length,
+         polygon.area,
+         polygon.length,
+         polygon.exterior.length],
+        dtype=np.float64)
     return result
 
 
@@ -410,17 +488,18 @@ def paths_to_polygons(paths, scale=None):
 
     Parameters
     -----------
-    paths: (n,) sequence, of (m,2) float, closed paths
-    scale: float, scale of drawing
+    paths : (n,) sequence
+        Of (m,2) float, closed paths
+    scale: float
+        Approximate scale of drawing for precision
 
     Returns
     -----------
-    polys: (p,) list of shapely.geometry.Polygons
-    valid: (n,) bool, whether input path was valid:
-                        valid.sum() == p
+    polys: (p,) list
+        shapely.geometry.Polygon
+        None
     """
-    polygons = []
-    valid = np.zeros(len(paths), dtype=np.bool)
+    polygons = [None] * len(paths)
     for i, path in enumerate(paths):
         if len(path) < 4:
             # since the first and last vertices are identical in
@@ -428,13 +507,68 @@ def paths_to_polygons(paths, scale=None):
             # non-zero area
             continue
         try:
-            polygons.append(repair_invalid(Polygon(path), scale))
-            valid[i] = True
+            polygons[i] = repair_invalid(Polygon(path), scale)
         except ValueError:
             # raised if a polygon is unrecoverable
             continue
+        except BaseException:
+            log.error('unrecoverable polygon', exc_info=True)
     polygons = np.array(polygons)
-    return polygons, valid
+    return polygons
+
+
+def sample(polygon, count, factor=1.5, max_iter=10):
+    """
+    Use rejection sampling to generate random points inside a
+    polygon.
+
+    Parameters
+    -----------
+    polygon : shapely.geometry.Polygon
+                Polygon that will contain points
+    count   : int
+                Number of points to return
+    factor  : float
+                How many points to test per loop
+                IE, count * factor
+    max_iter : int,
+                Maximum number of intersection loops
+                to run, total points sampled is
+                count * factor * max_iter
+
+    Returns
+    -----------
+    hit : (n, 2) float
+           Random points inside polygon
+           where n <= count
+    """
+    bounds = np.reshape(polygon.bounds, (2, 2))
+    extents = bounds.ptp(axis=0)
+
+    hit = []
+    hit_count = 0
+    per_loop = int(count * factor)
+
+    for i in range(max_iter):
+        # generate points inside polygons AABB
+        points = np.random.random((per_loop, 2))
+        points = (points * extents) + bounds[0]
+
+        # do the point in polygon test and append resulting hits
+        mask = vectorized.contains(polygon, *points.T)
+        hit.append(points[mask])
+
+        # keep track of how many points we've collected
+        hit_count += len(hit[-1])
+
+        # if we have enough points exit the loop
+        if hit_count > count:
+            break
+
+    # stack the hits into an (n,2) array and truncate
+    hit = np.vstack(hit)[:count]
+
+    return hit
 
 
 def repair_invalid(polygon, scale=None, rtol=.5):
@@ -466,7 +600,7 @@ def repair_invalid(polygon, scale=None, rtol=.5):
     if util.is_sequence(basic):
         basic = basic[np.argmax([i.area for i in basic])]
 
-    # check perimeter of result agains original perimeter
+    # check perimeter of result against original perimeter
     if basic.is_valid and np.isclose(basic.length,
                                      polygon.length,
                                      rtol=rtol):
@@ -486,7 +620,7 @@ def repair_invalid(polygon, scale=None, rtol=.5):
         if len(rings) == 1:
             # reconstruct a single polygon from the interior ring
             recon = Polygon(shell=rings[0]).buffer(distance)
-            # check perimeter of result agains original perimeter
+            # check perimeter of result against original perimeter
             if recon.is_valid and np.isclose(recon.length,
                                              polygon.length,
                                              rtol=rtol):
@@ -497,7 +631,7 @@ def repair_invalid(polygon, scale=None, rtol=.5):
     # if it returned multiple polygons check the largest
     if util.is_sequence(buffered):
         buffered = buffered[np.argmax([i.area for i in buffered])]
-    # check perimeter of result agains original perimeter
+    # check perimeter of result against original perimeter
     if buffered.is_valid and np.isclose(buffered.length,
                                         polygon.length,
                                         rtol=rtol):
