@@ -13,6 +13,7 @@ import copy
 
 from . import util
 from . import sample
+from . import caching
 from . import inertia
 from . import creation
 from . import transformations
@@ -102,10 +103,10 @@ class _Primitive(Trimesh):
             raise ValueError('Transformation matrix must be (4,4)!')
 
         if np.allclose(matrix, np.eye(4)):
-            log.debug('apply_tranform recieved identity matrix')
+            log.debug('apply_tranform received identity matrix')
             return
 
-        new_transform = np.dot(self.primitive.transform, matrix)
+        new_transform = np.dot(matrix, self.primitive.transform)
 
         self.primitive.transform = new_transform
 
@@ -127,14 +128,14 @@ class _PrimitiveAttributes(object):
         for key, value in kwargs.items():
             if key in defaults:
                 self._data[key] = util.convert_like(value, defaults[key])
-        # if configured as immutable, apply setting afer instantiation values
+        # if configured as immutable, apply setting after instantiation values
         # are set
         if 'mutable' in kwargs:
             self._mutable = bool(kwargs['mutable'])
 
     @property
     def __doc__(self):
-        # this is generated dynamically as the format operation can be suprisingly
+        # this is generated dynamically as the format operation can be surprisingly
         # slow and if generated in __init__ it is called a lot of times
         # when we didn't really need to generate it
 
@@ -204,7 +205,7 @@ class Cylinder(_Primitive):
                                               defaults,
                                               kwargs)
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def volume(self):
         """
         The analytic volume of the cylinder primitive.
@@ -216,7 +217,7 @@ class Cylinder(_Primitive):
         volume = (np.pi * self.primitive.radius ** 2) * self.primitive.height
         return volume
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def moment_inertia(self):
         """
         The analytic inertia tensor of the cylinder primitive.
@@ -367,7 +368,7 @@ class Sphere(_Primitive):
 
         return self.bounding_box
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def area(self):
         """
         Surface area of the current sphere primitive.
@@ -380,7 +381,7 @@ class Sphere(_Primitive):
         area = 4.0 * np.pi * (self.primitive.radius ** 2)
         return area
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def volume(self):
         """
         Volume of the current sphere primitive.
@@ -393,7 +394,7 @@ class Sphere(_Primitive):
         volume = (4.0 * np.pi * (self.primitive.radius ** 3)) / 3.0
         return volume
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def moment_inertia(self):
         """
         The analytic inertia tensor of the sphere primitive.
@@ -498,7 +499,7 @@ class Box(_Primitive):
         else:
             return False
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def volume(self):
         """
         Volume of the box Primitive.
@@ -534,7 +535,7 @@ class Extrusion(_Primitive):
         """
         super(Extrusion, self).__init__(*args, **kwargs)
 
-        # do the import here, so we fail early if Shapely isn't installed
+        # do the import here, fail early if Shapely isn't installed
         from shapely.geometry import Point
 
         defaults = {'polygon': Point([0, 0]).buffer(1.0),
@@ -545,7 +546,7 @@ class Extrusion(_Primitive):
                                               defaults,
                                               kwargs)
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def area(self):
         """
         The surface area of the primitive extrusion.
@@ -557,12 +558,13 @@ class Extrusion(_Primitive):
         area: float, surface area of 3D extrusion
         """
         # area of the sides of the extrusion
-        area = self.primitive.height * self.primitive.polygon.length
+        area = abs(self.primitive.height *
+                   self.primitive.polygon.length)
         # area of the two caps of the extrusion
         area += self.primitive.polygon.area * 2
         return area
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def volume(self):
         """
         The volume of the primitive extrusion.
@@ -573,7 +575,8 @@ class Extrusion(_Primitive):
         ----------
         volume: float, volume of 3D extrusion
         """
-        volume = self.primitive.polygon.area * self.primitive.height
+        volume = abs(self.primitive.polygon.area *
+                     self.primitive.height)
         return volume
 
     @property
@@ -593,8 +596,8 @@ class Extrusion(_Primitive):
 
     def slide(self, distance):
         """
-        Alter the transform of the current extrusion to slide it along its
-        extrude_direction vector
+        Alter the transform of the current extrusion to slide it
+        along its extrude_direction vector
 
         Parameters
         -----------
@@ -617,10 +620,20 @@ class Extrusion(_Primitive):
         buffered: Extrusion object
         """
         distance = float(distance)
-        buffered = Extrusion(transform=self.primitive.transform.copy(),
-                             polygon=self.primitive.polygon.buffer(distance),
-                             height=self.primitive.height + 2.0 * distance)
-        buffered.slide(-distance)
+
+        # start with current height
+        height = self.primitive.height
+        # if current height is negative offset by negative amount
+        height += np.sign(height) * 2.0 * distance
+
+        buffered = Extrusion(
+            transform=self.primitive.transform.copy(),
+            polygon=self.primitive.polygon.buffer(distance),
+            height=height)
+
+        # slide the stock along the axis
+        buffered.slide(-np.sign(height) * distance)
+
         return buffered
 
     def _create_mesh(self):
@@ -628,6 +641,7 @@ class Extrusion(_Primitive):
         mesh = creation.extrude_polygon(self.primitive.polygon,
                                         self.primitive.height)
         mesh.apply_transform(self.primitive.transform)
+
         self._cache['vertices'] = mesh.vertices
         self._cache['faces'] = mesh.faces
         self._cache['face_normals'] = mesh.face_normals
