@@ -1,6 +1,5 @@
 import numpy as np
 import collections
-import copy
 
 from .. import util
 from .. import units
@@ -11,121 +10,211 @@ from .. import transformations
 
 from .. import bounds as bounds_module
 
-from ..io import gltf
+from ..exchange import gltf
+from ..parent import Geometry
+
 from .transforms import TransformForest
 
 
-class Scene:
-    '''
-    A simple scene graph which can be rendered directly via pyglet/openGL,
-    or through other endpoints such as a raytracer.
-
-    Meshes and lights are added by name, which can then be moved by updating
-    transform in the transform tree.
-    '''
+class Scene(Geometry):
+    """
+    A simple scene graph which can be rendered directly via
+    pyglet/openGL or through other endpoints such as a
+    raytracer. Meshes are added by name, which can then be
+    moved by updating transform in the transform tree.
+    """
 
     def __init__(self,
                  geometry=None,
                  base_frame='world',
-                 metadata={}):
+                 metadata={},
+                 graph=None):
+        """
+        Create a new Scene object.
 
+        Parameters
+        -------------
+        geometry : Trimesh, Path2D, Path3D PointCloud or list
+          Geometry to initially add to the scene
+        base_frame : str or hashable
+          Name of base frame
+        metadata : dict
+          Any metadata about the scene
+        graph : TransformForest or None
+          A passed transform graph to use
+        """
         # mesh name : Trimesh object
         self.geometry = collections.OrderedDict()
 
-        # graph structure of instances
+        # create a new graph
         self.graph = TransformForest(base_frame=base_frame)
 
+        # create our cache
         self._cache = caching.Cache(id_function=self.md5)
 
+        # add passed geometry to scene
         self.add_geometry(geometry)
-        self.set_camera()
 
+        # hold metadata about the scene
         self.metadata = {}
         self.metadata.update(metadata)
 
+        if graph is not None:
+            # if we've been passed a graph override the default
+            self.graph = graph
+
+        self.camera = None
+
+    def apply_transform(self, transform):
+        raise NotImplementedError
+
     def add_geometry(self,
                      geometry,
-                     node_name=None):
-        '''
+                     node_name=None,
+                     geom_name=None,
+                     parent_node_name=None,
+                     transform=None):
+        """
         Add a geometry to the scene.
 
-        If the mesh has multiple transforms defined in its metadata, they will
-        all be copied into the TransformForest of the current scene automatically.
+        If the mesh has multiple transforms defined in its
+        metadata, they will all be copied into the
+        TransformForest of the current scene automatically.
 
         Parameters
         ----------
-        geometry: Trimesh, Path3D, or list of same
-        node_name: name in the scene graph
+        geometry : Trimesh, Path2D, Path3D PointCloud or list
+          Geometry to initially add to the scene
+        base_frame : str or hashable
+          Name of base frame
+        metadata : dict
+          Any metadata about the scene
+        graph : TransformForest or None
+          A passed transform graph to use
 
         Returns
         ----------
-        node_name: str, name of node in self.graph
-        '''
+        node_name : str
+          Name of node in self.graph
+        """
+
         if geometry is None:
             return
+        # PointCloud objects will look like a sequence
+        elif util.is_sequence(geometry):
+            # if passed a sequence add all elements
+            for value in geometry:
+                self.add_geometry(value)
+            return
 
-        # if passed a sequence call add_geometry on all elements
-        if util.is_sequence(geometry):
-            return [self.add_geometry(i) for i in geometry]
+        elif isinstance(geometry, dict):
+            # if someone passed us a dict of geometry
+            for key, value in geometry.items():
+                self.add_geometry(value, geom_name=key)
+            return
 
-        # default values for transforms and name
-        transforms = np.eye(4).reshape((-1, 4, 4))
-        geometry_name = 'geometry_' + str(len(self.geometry))
+        # get or create a name to reference the geometry by
+        if geom_name is not None:
+            # if name is passed use it
+            name = geom_name
+        elif 'name' in geometry.metadata:
+            # if name is in metadata use it
+            name = geometry.metadata['name']
+        elif 'file_name' in geometry.metadata:
+            name = geometry.metadata['file_name']
+        else:
+            # try to create a simple name
+            name = 'geometry_' + str(len(self.geometry))
 
-        # if object has metadata indicating different transforms or name
-        # use those values
-        if hasattr(geometry, 'metadata'):
-            if 'name' in geometry.metadata:
-                geometry_name = geometry.metadata['name']
-
-            if 'transforms' in geometry.metadata:
-                transforms = np.asanyarray(geometry.metadata['transforms'])
-                transforms = transforms.reshape((-1, 4, 4))
+        # if its already taken add a unique random string to it
+        if name in self.geometry:
+            name += ':' + util.unique_id().upper()
 
         # save the geometry reference
-        self.geometry[geometry_name] = geometry
+        self.geometry[name] = geometry
 
-        for i, transform in enumerate(transforms):
+        # create a unique node name if not passed
+        if node_name is None:
+            # a random unique identifier
+            unique = util.unique_id(increment=len(self.geometry))
+            # geometry name + UUID
+            node_name = name + '_' + unique.upper()
 
-            # if we haven't been passed a name to set in the graph
-            # use the geometry name plus an index
-            if node_name is None:
-                node_name = geometry_name + '_' + str(i)
+        if transform is None:
+            # create an identity transform from parent_node
+            transform = np.eye(4)
 
-            self.graph.update(frame_to=node_name,
-                              matrix=transform,
-                              geometry=geometry_name,
-                              geometry_flags={'visible': True})
+        self.graph.update(frame_to=node_name,
+                          frame_from=parent_node_name,
+                          matrix=transform,
+                          geometry=name,
+                          geometry_flags={'visible': True})
+        return node_name
 
     def md5(self):
-        '''
-        MD5 of scene, which will change when meshes or transforms are changed
+        """
+        MD5 of scene which will change when meshes or
+        transforms are changed
 
         Returns
         --------
         hashed: str, MD5 hash of scene
-        '''
+        """
+        # start with transforms hash
+        hashes = [self.graph.md5()]
+        for g in self.geometry.values():
+            if hasattr(g, 'md5'):
+                hashes.append(g.md5())
+            elif hasattr(g, 'tostring'):
+                hashes.append(str(hash(g.tostring())))
+            else:
+                # try to just straight up hash
+                # this may raise errors
+                hashes.append(str(hash(g)))
 
-        # get the MD5 of geometry and graph
-        data = [i.md5() for i in self.geometry.values()]
-        hashed = util.md5_object(np.append(data, self.graph.md5()))
+        md5 = util.md5_object(''.join(hashes))
 
-        return hashed
+        return md5
 
     @property
     def is_empty(self):
-        '''
+        """
         Does the scene have anything in it.
 
         Returns
         ----------
         is_empty: bool, True if nothing is in the scene
-        '''
+        """
 
         is_empty = len(self.geometry) == 0
         return is_empty
 
-    @util.cache_decorator
+    @property
+    def is_valid(self):
+        """
+        Is every geometry connected to the root node.
+
+        Returns
+        -----------
+        is_valid : bool
+          Does every geometry have a transform
+        """
+        if len(self.geometry) == 0:
+            return True
+
+        try:
+            referenced = {self.graph[i][1]
+                          for i in self.graph.nodes_geometry}
+        except BaseException:
+            # if connectivity to world frame is broken return false
+            return False
+
+        # every geometry is referenced
+        ok = referenced == set(self.geometry.keys())
+
+        return ok
+
+    @caching.cache_decorator
     def bounds_corners(self):
         """
         A list of points that represent the corners of the
@@ -157,115 +246,123 @@ class Scene:
                     corners_geom[geometry_name],
                     transform))
         # make corners numpy array
-        corners_inst = np.array(corners_inst, dtype=np.float64)
+        corners_inst = np.array(corners_inst,
+                                dtype=np.float64)
         return corners_inst
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def bounds(self):
-        '''
+        """
         Return the overall bounding box of the scene.
 
         Returns
         --------
         bounds: (2,3) float points for min, max corner
-        '''
+        """
         corners = self.bounds_corners
         bounds = np.array([corners.min(axis=0),
                            corners.max(axis=0)])
         return bounds
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def extents(self):
-        '''
+        """
         Return the axis aligned box size of the current scene.
 
         Returns
         ----------
         extents: (3,) float, bounding box sides length
-        '''
+        """
         return np.diff(self.bounds, axis=0).reshape(-1)
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def scale(self):
-        '''
+        """
         The approximate scale of the mesh
 
         Returns
         -----------
         scale: float, the mean of the bounding box edge lengths
-        '''
+        """
         scale = (self.extents ** 2).sum() ** .5
         return scale
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def centroid(self):
-        '''
+        """
         Return the center of the bounding box for the scene.
 
         Returns
         --------
         centroid: (3) float point for center of bounding box
-        '''
+        """
         centroid = np.mean(self.bounds, axis=0)
         return centroid
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def triangles(self):
-        '''
-        Return a correctly transformed polygon soup of the current scene.
+        """
+        Return a correctly transformed polygon soup of the
+        current scene.
 
         Returns
         ----------
         triangles: (n,3,3) float, triangles in space
-        '''
+        """
         triangles = collections.deque()
         triangles_node = collections.deque()
 
         for node_name in self.graph.nodes_geometry:
-
+            # which geometry does this node refer to
             transform, geometry_name = self.graph[node_name]
 
+            # get the actual potential mesh instance
             geometry = self.geometry[geometry_name]
             if not hasattr(geometry, 'triangles'):
                 continue
-
-            triangles.append(transformations.transform_points(
-                geometry.triangles.copy().reshape((-1, 3)),
-                transform))
-            triangles_node.append(np.tile(node_name, len(geometry.triangles)))
-
+            # append the (n, 3, 3) triangles to a sequence
+            triangles.append(
+                transformations.transform_points(
+                    geometry.triangles.copy().reshape((-1, 3)),
+                    matrix=transform))
+            # save the node names for each triangle
+            triangles_node.append(
+                np.tile(node_name,
+                        len(geometry.triangles)))
+        # save the resulting nodes to the cache
         self._cache['triangles_node'] = np.hstack(triangles_node)
         triangles = np.vstack(triangles).reshape((-1, 3, 3))
         return triangles
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def triangles_node(self):
-        '''
+        """
         Which node of self.graph does each triangle come from.
 
         Returns
         ---------
-        triangles_index: (len(self.triangles),) node name for each triangle
-        '''
+        triangles_index : (len(self.triangles),)
+            Node name for each triangle
+        """
         populate = self.triangles
         return self._cache['triangles_node']
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def geometry_identifiers(self):
-        '''
+        """
         Look up geometries by identifier MD5
 
         Returns
         ---------
         identifiers: dict, identifier md5: key in self.geometry
-        '''
+        """
         identifiers = {mesh.identifier_md5: name
                        for name, mesh in self.geometry.items()}
         return identifiers
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def duplicate_nodes(self):
-        '''
+        """
         Return a sequence of node keys of identical meshes.
 
         Will combine meshes duplicated by copying in space with different keys in
@@ -275,7 +372,7 @@ class Scene:
         -----------
         duplicates: (m) sequence of keys to self.nodes that represent
                      identical geometry
-        '''
+        """
         # if there is no geometry we can have no duplicate nodes
         if len(self.geometry) == 0:
             return []
@@ -288,6 +385,7 @@ class Scene:
         node_names = np.array(self.graph.nodes_geometry)
         # the geometry names for each node in the same order
         node_geom = np.array([self.graph[i][1] for i in node_names])
+
         # the mesh md5 for each node in the same order
         node_hash = np.array([mesh_hash[v] for v in node_geom])
 
@@ -298,29 +396,51 @@ class Scene:
         duplicates = [np.sort(node_names[g]).tolist() for g in node_groups]
         return duplicates
 
-    def set_camera(self, angles=None, distance=None, center=None):
-        '''
+    def set_camera(self,
+                   angles=None,
+                   distance=None,
+                   center=None,
+                   camera=None):
+        """
         Add a transform to self.graph for 'camera'
 
-        If arguments are not passed sane defaults will be figured out.
+        If arguments are not passed sane defaults will be figured
+        out which show the mesh roughly centered.
 
         Parameters
         -----------
-        angles:    (3,) float, initial euler angles in radians
-        distance:  float, distance away camera should be
-        center:    (3,) float, point camera should center on
+        angles : (3,) float
+          Initial euler angles in radians
+        distance : float
+          Distance from centroid
+        center : (3,) float
+          Point camera should be center on
+        camera : Camera object
+          Object that stores camera parameters
+        """
+        # passed camera object
+        if camera is not None:
+            self.graph.update(frame_from='camera',
+                              frame_to=self.graph.base_frame,
+                              matrix=camera.transform)
+            self.camera = camera
+            return
 
-        '''
+        # if no geometry nothing to set camera to
         if len(self.geometry) == 0:
             return
 
+        # use centroid if no center passed
         if center is None:
             center = self.centroid
+
+        # use scene AABB to set standoff distance
         if distance is None:
             # for a 60.0 degree horizontal FOV
             distance = ((self.extents.max() / 2) /
                         np.tan(np.radians(60.0) / 2.0))
 
+        # set with no rotation by default
         if angles is None:
             angles = np.zeros(3)
 
@@ -328,15 +448,17 @@ class Scene:
         translation[0:3, 3] = center
         # offset by a distance set by the model size
         # the FOV is set for the Y axis, we multiply by a lightly
-        # padded aspect ratio to make sure the model is in view initially
+        # padded aspect ratio to make sure the model is in view
         translation[2][3] += distance * 1.35
 
-        transform = np.dot(transformations.rotation_matrix(angles[0],
-                                                           [1, 0, 0],
-                                                           point=center),
-                           transformations.rotation_matrix(angles[1],
-                                                           [0, 1, 0],
-                                                           point=center))
+        transform = np.dot(transformations.rotation_matrix(
+            angles[0],
+            [1, 0, 0],
+            point=center),
+            transformations.rotation_matrix(
+            angles[1],
+            [0, 1, 0],
+            point=center))
         transform = np.dot(transform, translation)
 
         self.graph.update(frame_from='camera',
@@ -344,17 +466,18 @@ class Scene:
                           matrix=transform)
 
     def rezero(self):
-        '''
-        Move the current scene so that the AABB of the whole scene is centered
-        at the origin.
+        """
+        Move the current scene so that the AABB of the whole
+        scene is centered at the origin.
 
-        Does this by changing the base frame to a new, offset base frame.
-        '''
+        Does this by changing the base frame to a new, offset
+        base frame.
+        """
         if self.is_empty or np.allclose(self.centroid, 0.0):
             # early exit since what we want already exists
             return
 
-        # the transformation to move the overall scene to the AABB centroid
+        # the transformation to move the overall scene to AABB centroid
         matrix = np.eye(4)
         matrix[:3, 3] = -self.centroid
 
@@ -366,14 +489,14 @@ class Scene:
         self.graph.base_frame = new_base
 
     def dump(self):
-        '''
+        """
         Append all meshes in scene to a list of meshes.
 
         Returns
         ----------
         dumped: (n,) list, of Trimesh objects transformed to their
                            location the scene.graph
-        '''
+        """
         result = collections.deque()
 
         for node_name in self.graph.nodes_geometry:
@@ -384,56 +507,21 @@ class Scene:
             result.append(current)
         return np.array(result)
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def convex_hull(self):
-        '''
+        """
         The convex hull of the whole scene
 
         Returns
         ---------
         hull: Trimesh object, convex hull of all meshes in scene
-        '''
+        """
         points = util.vstack_empty([m.vertices for m in self.dump()])
         hull = convex.convex_hull(points)
         return hull
 
-    @util.cache_decorator
-    def bounding_box(self):
-        '''
-        An axis aligned bounding box for the current scene.
-
-        Returns
-        ----------
-        aabb: trimesh.primitives.Box object with transform and extents defined
-              to represent the axis aligned bounding box of the scene
-        '''
-        from .. import primitives
-        center = self.bounds.mean(axis=0)
-        aabb = primitives.Box(
-            transform=transformations.translation_matrix(center),
-            extents=self.extents,
-            mutable=False)
-        return aabb
-
-    @util.cache_decorator
-    def bounding_box_oriented(self):
-        '''
-        An oriented bounding box for the current mesh.
-
-        Returns
-        ---------
-        obb: trimesh.primitives.Box object with transform and extents defined
-             to represent the minimum volume oriented bounding box of the mesh
-        '''
-        from .. import primitives
-        to_origin, extents = bounds_module.oriented_bounds(self)
-        obb = primitives.Box(transform=np.linalg.inv(to_origin),
-                             extents=extents,
-                             mutable=False)
-        return obb
-
     def export(self, file_type=None):
-        '''
+        """
         Export a snapshot of the current scene.
 
         Parameters
@@ -447,8 +535,9 @@ class Scene:
                 meshes: list of meshes, encoded as per file_type
                 transforms: edge list of transforms, eg:
                              ((u, v, {'matrix' : np.eye(4)}))
-        '''
+        """
 
+        file_type = str(file_type).strip().lower()
         if file_type == 'gltf':
             return gltf.export_gltf(self)
         elif file_type == 'glb':
@@ -465,7 +554,8 @@ class Scene:
             file_type = {'Trimesh': 'ply',
                          'Path2D': 'dxf'}
 
-        # if the mesh has an export method use it, otherwise put the mesh
+        # if the mesh has an export method use it
+        # otherwise put the mesh
         # itself into the export object
         for geometry_name, geometry in self.geometry.items():
             if hasattr(geometry, 'export'):
@@ -478,8 +568,7 @@ class Scene:
                             break
                 else:
                     # if file_type is not a dict, try to export everything in the
-                    # scene as that value (probably a single string, like
-                    # 'ply')
+                    # the scene as that value like 'ply'
                     export_type = file_type
                 exported = {'data': geometry.export(file_type=export_type),
                             'file_type': export_type}
@@ -491,7 +580,7 @@ class Scene:
         return export
 
     def save_image(self, resolution=(1024, 768), **kwargs):
-        '''
+        """
         Get a PNG image of a scene.
 
         Parameters
@@ -502,34 +591,72 @@ class Scene:
         Returns
         -----------
         png: bytes, render of scene in PNG form
-        '''
-        from .viewer import render_scene
+        """
+        from ..viewer import render_scene
         png = render_scene(scene=self,
                            resolution=resolution,
                            **kwargs)
         return png
 
+    @property
+    def units(self):
+        """
+        Get the units for every model in the scene, and
+        raise a ValueError if there are mixed units.
+
+        Returns
+        -----------
+        units : str
+          Units for every model in the scene
+        """
+        existing = [i.units for i in self.geometry.values()]
+
+        if any(existing[0] != e for e in existing):
+            # if all of our geometry doesn't have the same units already
+            # this function will only do some hot nonsense
+            raise ValueError('models in scene have inconsistent units!')
+
+        return existing[0]
+
+    @units.setter
+    def units(self, value):
+        """
+        Set the units for every model in the scene without
+        converting any units just setting the tag.
+
+        Parameters
+        ------------
+        value : str
+          Value to set every geometry unit value to
+        """
+        for m in self.geometry.values():
+            m.units = value
+
     def convert_units(self, desired, guess=False):
-        '''
-        If geometry has units defined, convert them to new units.
+        """
+        If geometry has units defined convert them to new units.
 
         Returns a new scene with geometries and transforms scaled.
 
         Parameters
         ----------
-        units: str, target unit system. EG 'inches', 'mm', etc
-        '''
+        desired : str
+          Desired final unit system: 'inches', 'mm', etc.
+        guess : bool
+          Is the converter allowed to guess scale when models
+          don't have it specified in their metadata.
+
+        Returns
+        ----------
+        scaled : trimesh.Scene
+          Copy of scene with scaling applied and units set
+          for every model
+        """
         # if there is no geometry do nothing
         if len(self.geometry) == 0:
-            return self
+            return self.copy()
 
-        existing = [i.units for i in self.geometry.values()]
-        if any(existing[0] != e for e in existing):
-            # if all of our geometry doesn't have the same units already
-            # this function will only do some hot nonsense
-            raise ValueError('Models in scene have inconsistent units!')
-
-        current = existing[0]
+        current = self.units
         if current is None:
             # will raise ValueError if not in metadata
             # and not allowed to guess
@@ -545,20 +672,22 @@ class Scene:
         else:
             result = self.scaled(scale=scale)
 
-        for geometry in result.geometry.values():
-            geometry.units = desired
+        # apply the units to every geometry of the scaled result
+        result.units = desired
 
         return result
 
     def explode(self, vector=None, origin=None):
-        '''
+        """
         Explode a scene around a point and vector.
 
         Parameters
         -----------
-        vector: (3,) float, or float, explode in a direction or spherically
-        origin: (3,) float, point to explode around
-        '''
+        vector : (3,) float or float
+           Explode radially around a direction vector or spherically
+        origin : (3,) float
+          Point to explode around
+        """
         if origin is None:
             origin = self.centroid
         if vector is None:
@@ -588,86 +717,147 @@ class Scene:
             self.graph[node_name] = transform
 
     def scaled(self, scale):
-        '''
-        Return a copy of the current scene, with meshes and scene graph
+        """
+        Return a copy of the current scene, with meshes and scene
         transforms scaled to the requested factor.
 
         Parameters
         -----------
-        scale: float, factor to scale meshes and transforms by
-        '''
+        scale : float
+          Factor to scale meshes and transforms
+
+        Returns
+        -----------
+        scaled : trimesh.Scene
+          A copy of the current scene but scaled
+        """
         scale = float(scale)
-        scale_matrix = np.eye(4) * scale
+        # matrix for 2D scaling
+        scale_2D = np.eye(3) * scale
+        # matrix for 3D scaling
+        scale_3D = np.eye(4) * scale
 
-        transforms = np.array([self.graph[i][0]
-                               for i in self.graph.nodes_geometry])
-        geometries = np.array([self.graph[i][1]
-                               for i in self.graph.nodes_geometry])
+        # preallocate transforms and geometries
+        nodes = self.graph.nodes_geometry
+        transforms = np.zeros((len(nodes), 4, 4))
+        geometries = [None] * len(nodes)
 
+        # collect list of transforms
+        for i, node in enumerate(nodes):
+            transforms[i], geometries[i] = self.graph[node]
+
+        # result is a copy
         result = self.copy()
+        # remove all existing transforms
         result.graph.clear()
 
         for group in grouping.group(geometries):
+            # hashable reference to self.geometry
             geometry = geometries[group[0]]
+            # original transform from world to geometry
             original = transforms[group[0]]
-            new_geom = np.dot(scale_matrix, original)
+            # transform for geometry
+            new_geom = np.dot(scale_3D, original)
 
             if result.geometry[geometry].vertices.shape[1] == 2:
                 # if our scene is 2D only scale in 2D
-                result.geometry[geometry].apply_transform(np.eye(3) * scale)
+                result.geometry[geometry].apply_transform(scale_2D)
             else:
                 # otherwise apply the full transform
                 result.geometry[geometry].apply_transform(new_geom)
 
-            for node, t in zip(self.graph.nodes_geometry[group],
+            for node, T in zip(self.graph.nodes_geometry[group],
                                transforms[group]):
-                transform = util.multi_dot([scale_matrix,
-                                            t,
-                                            np.linalg.inv(new_geom)])
+                # generate the new transforms
+                transform = util.multi_dot(
+                    [scale_3D, T, np.linalg.inv(new_geom)])
+                # apply scale to translation
                 transform[:3, 3] *= scale
+                # update scene with new transforms
                 result.graph.update(frame_to=node,
                                     matrix=transform,
                                     geometry=geometry)
         return result
 
     def copy(self):
-        '''
+        """
         Return a deep copy of the current scene
 
         Returns
         ----------
-        copied: trimesh.Scene, copy of the current scene
-        '''
-        copied = copy.deepcopy(self)
+        copied : trimesh.Scene
+          Copy of the current scene
+        """
+        # use the geometries copy method to
+        # allow them to handle references to unpickle-able objects
+        geometry = {n: g.copy() for n, g in self.geometry.items()}
+        # create a new scene with copied geometry and graph
+        copied = Scene(geometry=geometry,
+                       graph=self.graph.copy())
         return copied
 
-    def show(self, **kwargs):
-        '''
-        Open a pyglet window to preview the current scene
+    def show(self, viewer=None, **kwargs):
+        """
+        Display the current scene.
 
         Parameters
         -----------
-        smooth: bool, turn on or off automatic smooth shading
-        '''
-        # this imports pyglet, and will raise an ImportError
-        # if pyglet is not available
-        from .viewer import SceneViewer
-        SceneViewer(self, **kwargs)
+        viewer: str 'gl':       open a pyglet window
+                str,'notebook': return ipython.display.HTML
+                None: automatically pick based on whether or not
+                          we are in an ipython notebook
+        smooth : bool
+          Turn on or off automatic smooth shading
+        """
+
+        if viewer is None:
+            # check to see if we are in a notebook or not
+            from ..viewer import in_notebook
+            viewer = ['gl', 'notebook'][int(in_notebook())]
+
+        if viewer == 'gl':
+            # this imports pyglet, and will raise an ImportError
+            # if pyglet is not available
+            from ..viewer import SceneViewer
+            return SceneViewer(self, **kwargs)
+        elif viewer == 'notebook':
+            from ..viewer import scene_to_notebook
+            return scene_to_notebook(self, **kwargs)
+        else:
+            raise ValueError('viewer must be "gl", "notebook", or None')
+
+    def __add__(self, other):
+        """
+        Concatenate the current scene with another scene or mesh.
+
+        Parameters
+        ------------
+        other : trimesh.Scene, trimesh.Trimesh, trimesh.Path
+           Other object to append into the result scene
+
+        Returns
+        ------------
+        appended : trimesh.Scene
+           Scene with geometry from both scenes
+        """
+        result = append_scenes([self, other],
+                               common=[self.graph.base_frame])
+        return result
 
 
 def split_scene(geometry):
-    '''
+    """
     Given a geometry, list of geometries, or a Scene
     return them as a single Scene object.
 
     Parameters
     ----------
-    geometry: splittable
+    geometry : splittable
 
     Returns
     ---------
     scene: trimesh.Scene
-    '''
+    """
     # already a scene, so return it
     if util.is_instance_named(geometry, 'Scene'):
         return geometry
@@ -685,9 +875,128 @@ def split_scene(geometry):
 
     # a single geometry so we are going to split
     split = collections.deque()
+
     metadata = {}
     for g in util.make_sequence(geometry):
         split.extend(g.split())
         metadata.update(g.metadata)
+
+    # if there is only one geometry in the mesh
+    # name it from the file name
+    if len(split) == 1 and 'file_name' in metadata:
+        split = {metadata['file_name']: split[0]}
+
     scene = Scene(split, metadata=metadata)
+
     return scene
+
+
+def append_scenes(iterable, common=['world']):
+    """
+    Concatenate multiple scene objects into one scene.
+
+    Parameters
+    -------------
+    iterable : (n,) Trimesh or Scene
+       Geometries that should be appended
+    common : (n,) str
+       Nodes that shouldn't be remapped
+
+    Returns
+    ------------
+    result : trimesh.Scene
+       Scene containing all geometry
+    """
+    if isinstance(iterable, Scene):
+        return iterable
+
+    # save geometry in dict
+    geometry = {}
+    # save transforms as edge tuples
+    edges = []
+
+    # nodes which shouldn't be remapped
+    common = set(common)
+    # nodes which are consumed and need to be remapped
+    consumed = set()
+
+    def node_remap(node):
+        """
+        Remap node to new name if necessary
+
+        Parameters
+        -------------
+        node : hashable
+           Node name in original scene
+
+        Returns
+        -------------
+        name : hashable
+           Node name in concatenated scene
+        """
+
+        # if we've already remapped a node use it
+        if node in map_node:
+            return map_node[node]
+
+        # if a node is consumed and isn't one of the nodes
+        # we're going to hold common between scenes remap it
+        if node not in common and node in consumed:
+            name = str(node) + '-' + util.unique_id().upper()
+            map_node[node] = name
+            node = name
+
+        # keep track of which nodes have been used
+        # in the current scene
+        current.add(node)
+        return node
+
+    # loop through every geometry
+    for s in iterable:
+        # allow Trimesh/Path2D geometry to be passed
+        if hasattr(s, 'scene'):
+            s = s.scene()
+        # if we don't have a scene raise an exception
+        if not isinstance(s, Scene):
+            raise ValueError('{} is not a scene!'.format(
+                type(s).__name__))
+
+        # remap geometries if they have been consumed
+        map_geom = {}
+        for k, v in s.geometry.items():
+            # if a geometry already exists add a UUID to the name
+            if k in geometry:
+                name = str(k) + '-' + util.unique_id().upper()
+            else:
+                name = k
+            # store name mapping
+            map_geom[k] = name
+            # store geometry with new name
+            geometry[name] = v
+
+        # remap nodes and edges so duplicates won't
+        # stomp all over each other
+        map_node = {}
+        # the nodes used in this scene
+        current = set()
+        for a, b, attr in s.graph.to_edgelist():
+            # remap node names from local names
+            a, b = node_remap(a), node_remap(b)
+            # remap geometry keys
+            # if key is not in map_geom it means one of the scenes
+            # referred to geometry that doesn't exist
+            # rather than crash here we ignore it as the user
+            # possibly intended to add in geometries back later
+            if 'geometry' in attr and attr['geometry'] in map_geom:
+                attr['geometry'] = map_geom[attr['geometry']]
+            # save the new edge
+            edges.append((a, b, attr))
+        # mark nodes from current scene as consumed
+        consumed.update(current)
+
+    # add all data to a new scene
+    result = Scene()
+    result.graph.from_edgelist(edges)
+    result.geometry.update(geometry)
+
+    return result
