@@ -1,15 +1,15 @@
 import time
+import copy
 import collections
 
 import numpy as np
 import networkx as nx
 
-from .. import util
 from .. import caching
 from .. import transformations
 
 
-class TransformForest:
+class TransformForest(object):
 
     def __init__(self, base_frame='world'):
         self.transforms = EnforcedForest()
@@ -40,8 +40,7 @@ class TransformForest:
         axis:        (3) array
         angle:       float, radians
         translation: (3) array
-
-        geometry: Geometry object name
+        geometry : Geometry object name
         """
         if frame_from is None:
             frame_from = self.base_frame
@@ -73,6 +72,20 @@ class TransformForest:
         result = str(int(self._updated * 1000)) + str(self.base_frame)
         return result
 
+    def copy(self):
+        """
+        Return a copy of the current TransformForest
+
+        Returns
+        ------------
+        copied: TransformForest
+        """
+        copied = TransformForest()
+        copied.base_frame = copy.deepcopy(self.base_frame)
+        copied.transforms = copy.deepcopy(self.transforms)
+
+        return copied
+
     def to_flattened(self, base_frame=None):
         """
         Export the current transform graph as a flattened
@@ -128,17 +141,59 @@ class TransformForest:
         -------
         edgelist: (n,) list of tuples
         """
-        # wrapped in a list for nx 2.0
-        export = list(nx.to_edgelist(self.transforms))
-        for e in export:
-            e[2]['matrix'] = np.array(e[2]['matrix']).tolist()
+        # save cleaned edges
+        export = []
+        # loop through (node, node, edge attributes)
+        for edge in nx.to_edgelist(self.transforms):
+            a, b, c = edge
+            # geometry is a node property but save it to the
+            # edge so we don't need two dictionaries
+            if 'geometry' in self.transforms.node[b]:
+                c['geometry'] = self.transforms.node[b]['geometry']
+            # save the matrix as a float list
+            c['matrix'] = np.asanyarray(c['matrix'],
+                                        dtype=np.float64).tolist()
+            export.append((a, b, c))
         return export
 
-    def load(self, edgelist):
-        for edge in edgelist:
-            self.transforms.add_edge(edge[0], edge[1], **edge[2])
+    def from_edgelist(self, edges, strict=True):
+        """
+        Load transform data from an edge list into the current
+        scene graph.
 
-    @util.cache_decorator
+        Parameters
+        -------------
+        edgelist : (n,) tuples
+            (node_a, node_b, {key: value})
+        strict : bool
+            If true, raise a ValueError when a
+            malformed edge is passed in a tuple.
+        """
+        # loop through each edge
+        for edge in edges:
+            # edge contains attributes
+            if len(edge) == 3:
+                self.update(edge[1], edge[0], **edge[2])
+            # edge just contains nodes
+            elif len(edge) == 2:
+                self.update(edge[1], edge[0])
+            # edge is broken
+            elif strict:
+                raise ValueError('edge incorrect shape: {}'.format(str(edge)))
+
+    def load(self, edgelist):
+        """
+        Load transform data from an edge list into the current
+        scene graph.
+
+        Parameters
+        -------------
+        edgelist : (n,) tuples
+            (node_a, node_b, {key: value})
+        """
+        self.from_edgelist(edgelist, strict=True)
+
+    @caching.cache_decorator
     def nodes(self):
         """
         A list of every node in the graph.
@@ -150,7 +205,7 @@ class TransformForest:
         nodes = np.array(list(self.transforms.nodes()))
         return nodes
 
-    @util.cache_decorator
+    @caching.cache_decorator
     def nodes_geometry(self):
         """
         The nodes in the scene graph with geometry attached.
@@ -159,16 +214,11 @@ class TransformForest:
         ------------
         nodes_geometry: (m,) array, of node names
         """
-        nodes = self.transforms.nodes()
-        if len(nodes) == 0:
-            return np.array([])
 
-        nodes_geometry = []
-        for n in nodes:
-            if (n in self.transforms.node and
-                    'geometry' in self.transforms.node[n]):
-                nodes_geometry.append(n)
-        return np.array(nodes_geometry)
+        nodes = np.array([n for n in self.transforms.nodes()
+                          if 'geometry' in self.transforms.node[n]])
+
+        return nodes
 
     def get(self,
             frame_to,
@@ -312,7 +362,9 @@ class EnforcedForest(nx.DiGraph):
                         'Multiple edge path exists between nodes!')
                 self.disconnect_path(path)
                 changed = True
-            except (nx.NetworkXError, nx.NetworkXNoPath, nx.NetworkXException):
+            except (nx.NetworkXError,
+                    nx.NetworkXNoPath,
+                    nx.NetworkXException):
                 pass
         self._undirected.add_edge(u, v)
         super(self.__class__, self).add_edge(u, v, *args, **kwargs)
@@ -353,7 +405,7 @@ class EnforcedForest(nx.DiGraph):
         elif self.has_edge(v, u):
             direction = -1
         else:
-            raise ValueError('Edge doesnt exist!')
+            raise ValueError('Edge does not exist!')
         data = self.get_edge_data(*[u, v][::direction])
         return data, direction
 
