@@ -21,16 +21,18 @@ import zipfile
 import collections
 
 from sys import version_info
-from functools import wraps
 
 # a flag we can check elsewhere for Python 3
 PY3 = version_info.major >= 3
 if PY3:
     # for type checking
     basestring = str
+    # Python 3
     from io import BytesIO, StringIO
 else:
+    # Python 2
     from StringIO import StringIO
+    BytesIO = StringIO
 
 log = logging.getLogger('trimesh')
 log.addHandler(logging.NullHandler())
@@ -44,53 +46,63 @@ TOL_ZERO = np.finfo(np.float64).resolution * 100
 TOL_MERGE = 1e-8
 
 
-def unitize(points, check_valid=False, threshold=None):
+def unitize(vectors,
+            check_valid=False,
+            threshold=None):
     """
-    Turn a list of vectors into a list of unit vectors.
+    Unitize a vector or an array or row- vectors.
 
     Parameters
     ---------
-    points:       (n,m) or (j) input array of vectors.
-                  For 1D arrays, points is treated as a single vector
-                  For 2D arrays, each row is treated as a vector
-
-    check_valid:  boolean, if True enables valid output and checking
-
-    threshold:    float, cutoff to be considered zero.
-
+    vectors : (n,m) or (j) float
+       Vector or vectors to be unitized
+    check_valid :  bool
+       If set, will return mask of nonzero vectors
+    threshold : float
+       Cutoff for a value to be considered zero.
 
     Returns
     ---------
-    unit_vectors: (n,m) or (j) length array of unit vectors
-
-    valid:        (n) boolean array, output only if check_valid.
-                   True for all valid (nonzero length) vectors, thus m=sum(valid)
+    unit :  (n,m) or (j) float
+       Input vectors but unitized
+    valid : (n,) bool or bool
+        Mask of nonzero vectors returned if `check_valid`
     """
-    points = np.asanyarray(points)
-    axis = len(points.shape) - 1
-    length = np.sum(points ** 2, axis=axis) ** .5
+    # make sure we have a numpy array
+    vectors = np.asanyarray(vectors)
 
-    if is_sequence(length):
-        length[np.isnan(length)] = 0.0
+    # allow user to set zero threshold
     if threshold is None:
         threshold = TOL_ZERO
 
-    if check_valid:
-        # make sure lengths are greater than zero
-        valid = np.logical_not(np.isclose(length,
-                                          0.0,
-                                          rtol=0.0,
-                                          atol=threshold))
-        if axis == 1:
-            unit_vectors = (points[valid].T / length[valid]).T
-        elif len(points.shape) == 1 and valid:
-            unit_vectors = points / length
+    if len(vectors.shape) == 2:
+        # for (m, d) arrays take the per- row unit vector
+        # using sqrt and avoiding exponents is slightly faster
+        # also dot with ones is faser than .sum(axis=1)
+        norm = np.sqrt(np.dot(vectors * vectors,
+                              np.ones(vectors.shape[1])))
+        # non-zero norms
+        valid = norm > threshold
+        # in-place reciprocal of nonzero norms
+        norm[valid] **= -1
+        # tile reciprocal of norm
+        tiled = np.tile(norm, (vectors.shape[1], 1)).T
+        # multiply by reciprocal of norm
+        unit = vectors * tiled
+    elif len(vectors.shape) == 1:
+        # treat 1D arrays as a single vector
+        norm = np.sqrt((vectors * vectors).sum())
+        valid = norm > threshold
+        if valid:
+            unit = vectors / norm
         else:
-            unit_vectors = np.array([])
-        return unit_vectors, valid
+            unit = vectors.copy()
     else:
-        unit_vectors = (points.T / length).T
-    return unit_vectors
+        raise ValueError('vectors must be (n, ) or (n, d)!')
+
+    if check_valid:
+        return unit[valid], valid
+    return unit
 
 
 def euclidean(a, b):
@@ -99,17 +111,19 @@ def euclidean(a, b):
 
     Parameters
     ------------
-    a: (n,) float, vector A
-    b: (n,) float, vector B
+    a : (n,) float
+       First vector
+    b : (n,) float
+       Second vector
 
     Returns
     ------------
-    distance: float, euclidean distance between A and B
+    distance : float
+        Euclidean distance between A and B
     """
     a = np.asanyarray(a, dtype=np.float64)
     b = np.asanyarray(b, dtype=np.float64)
-    distance = np.sum((a - b) ** 2) ** .5
-    return distance
+    return np.sqrt(((a - b) ** 2).sum())
 
 
 def is_file(obj):
@@ -118,11 +132,13 @@ def is_file(obj):
 
     Parameters
     ------------
-    obj: object to be checked
+    obj : object
+       Any object type to be checked
 
     Returns
     -----------
-    is_file: bool, True if object is a file
+    is_file : bool
+        True if object is a file
     """
     return hasattr(obj, 'read') or hasattr(obj, 'write')
 
@@ -133,11 +149,13 @@ def is_string(obj):
 
     Parameters
     ------------
-    obj: object to be checked
+    obj : object
+       Any object type to be checked
 
     Returns
     ------------
-    is_string: bool, True if obj is a string
+    is_string : bool
+        True if obj is a string
     """
     return isinstance(obj, basestring)
 
@@ -150,11 +168,13 @@ def is_none(obj):
 
     Parameters
     -------------
-    obj: object to be checked
+    obj : object
+      Any object type to be checked
 
     Returns
     -------------
-    is_none: bool, True if obj is None
+    is_none : bool
+        True if obj is None or numpy None-like
     """
     if obj is None:
         return True
@@ -171,11 +191,13 @@ def is_sequence(obj):
 
     Parameters
     -------------
-    obj: object to be checked
+    obj : object
+      Any object type to be checked
 
     Returns
     -------------
-    is_sequence: bool, True if object is sequence
+    is_sequence : bool
+        True if object is sequence
     """
     seq = (not hasattr(obj, "strip") and
            hasattr(obj, "__getitem__") or
@@ -186,6 +208,9 @@ def is_sequence(obj):
                                                        set,
                                                        basestring))
 
+    # PointCloud objects can look like an array but are not
+    seq = seq and type(obj).__name__ not in ['PointCloud']
+
     # numpy sometimes returns objects that are single float64 values
     # but sure look like sequences, so we check the shape
     if hasattr(obj, 'shape'):
@@ -204,10 +229,11 @@ def is_shape(obj, shape):
 
     Parameters
     ---------
-    obj:   np.ndarray to check the shape of
-    shape: list or tuple of shape.
-           Any negative term will be considered a wildcard
-           Any tuple term will be evaluated as an OR
+    obj :   np.ndarray
+       Array to check the shape on
+    shape : list or tuple
+       Any negative term will be considered a wildcard
+       Any tuple term will be evaluated as an OR
 
     Returns
     ---------
@@ -215,24 +241,24 @@ def is_shape(obj, shape):
 
     Examples
     ------------------------
-    In [1]: a = np.random.random((100,3))
+    In [1]: a = np.random.random((100, 3))
 
     In [2]: a.shape
     Out[2]: (100, 3)
 
-    In [3]: trimesh.util.is_shape(a, (-1,3))
+    In [3]: trimesh.util.is_shape(a, (-1, 3))
     Out[3]: True
 
-    In [4]: trimesh.util.is_shape(a, (-1,3,5))
+    In [4]: trimesh.util.is_shape(a, (-1, 3, 5))
     Out[4]: False
 
-    In [5]: trimesh.util.is_shape(a, (100,-1))
+    In [5]: trimesh.util.is_shape(a, (100, -1))
     Out[5]: True
 
-    In [6]: trimesh.util.is_shape(a, (-1,(3,4)))
+    In [6]: trimesh.util.is_shape(a, (-1, (3, 4)))
     Out[6]: True
 
-    In [7]: trimesh.util.is_shape(a, (-1,(4,5)))
+    In [7]: trimesh.util.is_shape(a, (-1, (4, 5)))
     Out[7]: False
     """
 
@@ -284,11 +310,13 @@ def make_sequence(obj):
 
     Parameters
     --------------
-    obj: object to be made a sequence
+    obj : object
+      An object to be made a sequence
 
     Returns
     --------------
-    as_sequence: (n,) sequence containing input
+    as_sequence : (n,) sequence
+       Contains input value
     """
     if is_sequence(obj):
         return np.array(list(obj))
@@ -296,56 +324,97 @@ def make_sequence(obj):
         return np.array([obj])
 
 
-def vector_hemisphere(vectors):
+def vector_hemisphere(vectors, return_sign=False):
     """
-    For a set of 3D vectors alter the sign so they are all in the upper
-    hemisphere.
+    For a set of 3D vectors alter the sign so they are all in the
+    upper hemisphere.
 
-    If the vector lies on the plane, all vectors with negative Y will be reversed.
-    If the vector has a zero Z and Y value, vectors with a negative X value
-    will be reversed
+    If the vector lies on the plane all vectors with negative Y
+    will be reversed.
+
+    If the vector has a zero Z and Y value vectors with a
+    negative X value will be reversed.
 
     Parameters
     ----------
-    vectors: (n,3) float, set of vectors
+    vectors : (n,3) float
+      Input vectors
+    return_sign : bool
+      Return the sign mask or not
 
     Returns
     ----------
-    oriented: (n,3) float, set of vectors with same magnitude but all
-                           pointing in the same hemisphere.
+    oriented: (n, 3) float
+       Vectors with same magnitude as source
+       but possibly reversed to ensure all vectors
+       are in the same hemisphere.
+    sign : (n,) float
+
 
     """
+    # vectors as numpy array
     vectors = np.asanyarray(vectors, dtype=np.float64)
-    if not is_shape(vectors, (-1, 3)):
-        raise ValueError('Vectors must be (n,3)!')
 
-    neg = vectors < -TOL_ZERO
-    zero = np.logical_not(np.logical_or(neg, vectors > TOL_ZERO))
+    if is_shape(vectors, (-1, 2)):
+        # 2D vector case
+        # check the Y value and reverse vector
+        # direction if negative.
+        negative = vectors < -TOL_ZERO
+        zero = np.logical_not(
+            np.logical_or(negative, vectors > TOL_ZERO))
 
-    # move all                          negative Z to positive
-    # then for zero Z vectors, move all negative Y to positive
-    # then for zero Y vectors, move all negative X to positive
+        signs = np.ones(len(vectors), dtype=np.float64)
+        # negative Y values are reversed
+        signs[negative[:, 1]] = -1.0
 
-    signs = np.ones(len(vectors), dtype=np.float64)
+        # zero Y and negative X are reversed
+        signs[np.logical_and(zero[:, 1], negative[:, 0])] = -1.0
 
-    # all vectors with negative Z values
-    signs[neg[:, 2]] = -1.0
-    # all on-plane vectors with negative Y values
-    signs[np.logical_and(zero[:, 2], neg[:, 1])] = -1.0
-    # all on-plane vectors with zero Y values and negative X values
-    signs[np.logical_and(
-        np.logical_and(
-            zero[:, 2],
-            zero[:, 1]),
-        neg[:, 0])] = -1.0
+    elif is_shape(vectors, (-1, 3)):
+        # 3D vector case
+        negative = vectors < -TOL_ZERO
+        zero = np.logical_not(
+            np.logical_or(negative, vectors > TOL_ZERO))
+        # move all                          negative Z to positive
+        # then for zero Z vectors, move all negative Y to positive
+        # then for zero Y vectors, move all negative X to positive
+        signs = np.ones(len(vectors), dtype=np.float64)
+        # all vectors with negative Z values
+        signs[negative[:, 2]] = -1.0
+        # all on-plane vectors with negative Y values
+        signs[np.logical_and(zero[:, 2], negative[:, 1])] = -1.0
+        # all on-plane vectors with zero Y values
+        # and negative X values
+        signs[np.logical_and(np.logical_and(zero[:, 2],
+                                            zero[:, 1]),
+                             negative[:, 0])] = -1.0
 
+    else:
+        raise ValueError('vectors must be (n,3)!')
+
+    # apply the signs to the vectors
     oriented = vectors * signs.reshape((-1, 1))
+
+    if return_sign:
+        return oriented, signs
+
     return oriented
 
 
 def vector_to_spherical(cartesian):
     """
-    Convert a set of cartesian points to (n,2) spherical vectors
+    Convert a set of cartesian points to (n,2) spherical unit
+    vectors.
+
+    Parameters
+    ------------
+    cartesian : (n, 3) float
+       Points in space
+
+    Returns
+    ------------
+    spherical : (n, 2) float
+       Angles, in radians
     """
     cartesian = np.asanyarray(cartesian, dtype=np.float64)
     if not is_shape(cartesian, (-1, 3)):
@@ -364,11 +433,20 @@ def vector_to_spherical(cartesian):
 def spherical_to_vector(spherical):
     """
     Convert a set of (n,2) spherical vectors to (n,3) vectors
+
+    Parameters
+    -----------
+    spherical : (n , 2) float
+       Angles, in radians
+
+    Returns
+    -----------
+    vectors : (n, 3) float
+      Unit vectors
     """
     spherical = np.asanyarray(spherical, dtype=np.float64)
     if not is_shape(spherical, (-1, 2)):
-        raise ValueError(
-            'Spherical vectors must be passed as an (n,2) set of angles!')
+        raise ValueError('spherical coordinates must be (n, 2)!')
 
     theta, phi = spherical.T
     st, ct = np.sin(theta), np.cos(theta)
@@ -385,11 +463,13 @@ def pairwise(iterable):
 
     Parameters
     -----------
-    iterable: flat list
+    iterable : (m, ) list
+       A sequence of values
 
     Returns
     -----------
-    pairs: (n,) seq of (2,) pairs of items
+    pairs: (n, 2)
+      Pairs of sequential values
 
     Example
     -----------
@@ -412,7 +492,8 @@ def pairwise(iterable):
     import itertools
     a, b = itertools.tee(iterable)
     # pop the first element of the second item
-    popped = next(b)
+    next(b)
+
     return zip(a, b)
 
 
@@ -443,7 +524,7 @@ def diagonal_dot(a, b):
     Same as np.diag(np.dot(a, b.T)) but without the monstrous
     intermediate matrix.
 
-    Also equivilant to:
+    Also equivalent to:
     np.einsum('ij,ij->i', a, b)
 
     Parameters
@@ -573,13 +654,22 @@ def multi_dict(pairs):
     return result
 
 
-def tolist_dict(data):
-    def tolist(item):
-        if hasattr(item, 'tolist'):
-            return item.tolist()
-        else:
-            return item
-    result = {k: tolist(v) for k, v in data.items()}
+def tolist(data):
+    """
+    Ensure that any arrays or dicts passed containing
+    numpy arrays are properly converted to lists
+
+    Parameters
+    -----------------
+    data : any
+      Usually a dict with some numpy arrays as values
+
+    Returns
+    ------------
+    result : any
+      JSON- serializable version of data
+    """
+    result = json.loads(jsonify(data))
     return result
 
 
@@ -629,7 +719,7 @@ def decimal_to_digits(decimal, min_digits=None):
     Parameters
     -----------
     decimal:    float
-    min_digits: int, minumum number of digits to return
+    min_digits: int, minimum number of digits to return
 
     Returns
     -----------
@@ -684,31 +774,16 @@ def md5_object(obj):
     md5: str, MD5 hash
     """
     hasher = hashlib.md5()
-    hasher.update(obj)
+    if isinstance(obj, basestring) and PY3:
+        # in python3 convert strings to bytes before hashing
+        hasher.update(obj.encode('utf-8'))
+    else:
+        hasher.update(obj)
+
     md5 = hasher.hexdigest()
     return md5
 
 
-def md5_array(array, digits=5):
-    """
-    Take the MD5 of an array when considering the specified number of digits.
-
-    Parameters
-    ---------
-    array:  numpy array
-    digits: int, number of digits to account for in the MD5
-
-    Returns
-    ---------
-    md5: str, md5 hash of input
-    """
-    digits = int(digits)
-    array = np.asanyarray(array, dtype=np.float64).reshape(-1)
-    as_int = (array * 10 ** digits).astype(np.int64)
-    md5 = md5_object(as_int.tostring(order='C'))
-    return md5
-
-
 def attach_to_log(level=logging.DEBUG,
                   handler=None,
                   loggers=None,
@@ -777,30 +852,6 @@ def attach_to_log(level=logging.DEBUG,
     np.set_printoptions(precision=5, suppress=True)
 
 
-def cache_decorator(function):
-    """
-    A decorator for methods of classes.
-
-    If the object contains `self.cache`, this function
-    will check the values in self.cache for this wrapped function
-    by name, and if it is already in the cache it will return that
-    value rather than evaluating the function again.
-    """
-    @wraps(function)
-    def get_cached(*args, **kwargs):
-        self = args[0]
-        name = function.__name__
-        if not (name in self._cache):
-            tic = time.time()
-            self._cache[name] = function(*args, **kwargs)
-            toc = time.time()
-            log.debug('%s was not in cache, executed in %.6f',
-                      name,
-                      toc - tic)
-        return self._cache[name]
-    return property(get_cached)
-
-
 def stack_lines(indices):
     """
     Stack a list of values that represent a polyline into
@@ -847,29 +898,39 @@ def stack_lines(indices):
 
 def append_faces(vertices_seq, faces_seq):
     """
-    Given a sequence of zero- indexed faces and vertices,
-    combine them into a single (n,3) list of faces and (m,3) vertices
+    Given a sequence of zero- indexed faces and vertices
+    combine them into a single array of faces and
+    a single array of vertices.
 
     Parameters
-    ---------
-    vertices_seq: (n) sequence of (m,d) vertex arrays
-    faces_seq     (n) sequence of (p,j) faces, zero indexed
-                  and referencing their counterpoint vertices
+    -----------
+    vertices_seq : (n, ) sequence of (m, d) float
+      Multiple arrays of verticesvertex arrays
+    faces_seq : (n, ) sequence of (p, j) int
+      Zero indexed faces for matching vertices
 
     Returns
     ----------
-    vertices: (i, d) float, vertices
-    faces:    (j, 3) int, faces
+    vertices : (i, d) float
+      Points in space
+    faces : (j, 3) int
+      Reference vertex indices
     """
+    # the length of each vertex array
     vertices_len = np.array([len(i) for i in vertices_seq])
+    # how much each group of faces needs to be offset
     face_offset = np.append(0, np.cumsum(vertices_len)[:-1])
 
+    new_faces = []
     for offset, faces in zip(face_offset, faces_seq):
-        if len(faces) > 0:
-            faces += offset
-
+        if len(faces) == 0:
+            continue
+        # apply the index offset
+        new_faces.append(faces + offset)
+    # stack to clean (n, 3) float
     vertices = vstack_empty(vertices_seq)
-    faces = vstack_empty(faces_seq)
+    # stack to clean (n, 3) int
+    faces = vstack_empty(new_faces)
 
     return vertices, faces
 
@@ -880,20 +941,30 @@ def array_to_string(array,
                     digits=8,
                     value_format='{}'):
     """
-    Convert a 1 or 2D array into a string with a specified number of digits
-    and delimiter.
+    Convert a 1 or 2D array into a string with a specified number
+    of digits and delimiter. The reason this exists is that the
+    basic numpy array to string conversions are surprisingly bad.
 
     Parameters
     ----------
-    array:     (n,) or (n,d) float/int, array to be converted
-               If (n,) only column delimiter will be used
-    col_delim: str, what string should separate values in a column
-    row_delim: str, what string should separate values in a row
-    digits:    int, how many digits should floating point numbers include
+    array : (n,) or (n, d) float or int
+       Data to be converted
+       If shape is (n,) only column delimiter will be used
+    col_delim : str
+      What string should separate values in a column
+    row_delim : str
+      What string should separate values in a row
+    digits : int
+      How many digits should floating point numbers include
+    value_format : str
+       Format string for each value or sequence of values
+       If multiple values per value_format it must divide
+       into array evenly.
 
     Returns
     ----------
-    formatted: str, string representation of original array
+    formatted : str
+       String representation of original array
     """
     # convert inputs to correct types
     array = np.asanyarray(array)
@@ -918,7 +989,7 @@ def array_to_string(array,
         format_str = value_format.replace(
             '{}', '{:.' + str(digits) + 'f}') + col_delim
     else:
-        raise ValueError('dtype %s not convertable!',
+        raise ValueError('dtype %s not convertible!',
                          array.dtype.name)
 
     # length of extra delimiters at the end
@@ -926,7 +997,7 @@ def array_to_string(array,
     # if we have a 2D array add a row delimiter
     if len(array.shape) == 2:
         format_str *= array.shape[1]
-        # cut off the last column delimeter and add a row delimiter
+        # cut off the last column delimiter and add a row delimiter
         format_str = format_str[:-len(col_delim)] + row_delim
         end_junk = len(row_delim)
 
@@ -949,17 +1020,21 @@ def array_to_encoded(array, dtype=None, encoding='base64'):
     Export a numpy array to a compact serializable dictionary.
 
     Parameters
-    ---------
-    array: numpy array
-    dtype: optional, what dtype should array be encoded with.
-    encoding: str, 'base64' or 'binary'
+    ------------
+    array : array
+      Any numpy array
+    dtype : str or None
+      Optional dtype to encode array
+    encoding : str
+      'base64' or 'binary'
 
     Returns
     ---------
-    encoded: dict with keys:
-                 dtype: string of dtype
-                 shape: int tuple of shape
-                 base64: base64 encoded string of flat array
+    encoded : dict
+      Has keys:
+      'dtype':  str, of dtype
+      'shape':  tuple of shape
+      'base64': str, base64 encoded string
     """
     array = np.asanyarray(array)
     shape = array.shape
@@ -984,15 +1059,26 @@ def array_to_encoded(array, dtype=None, encoding='base64'):
 
 def decode_keys(store, encoding='utf-8'):
     """
-    If a dictionary has keys that are bytes encode them (utf-8 default)
+    If a dictionary has keys that are bytes decode them to a str.
 
     Parameters
     ---------
-    store: dict
+    store : dict
+      Dictionary with data
 
     Returns
     ---------
-    store: dict, with same data and if keys were bytes they have been encoded
+    result : dict
+      Values are untouched but keys that were bytes
+      are converted to ASCII strings.
+
+    Example
+    -----------
+    In [1]: d
+    Out[1]: {1020: 'nah', b'hi': 'stuff'}
+
+    In [2]: trimesh.util.decode_keys(d)
+    Out[2]: {1020: 'nah', 'hi': 'stuff'}
     """
     keys = store.keys()
     for key in keys:
@@ -1009,12 +1095,14 @@ def encoded_to_array(encoded):
     Turn a dictionary with base64 encoded strings back into a numpy array.
 
     Parameters
-    ----------
-    encoded: dict with keys:
-                 dtype: string of dtype
-                 shape: int tuple of shape
-                 base64: base64 encoded string of flat array
-                 binary:  decode result coming from numpy.tostring
+    ------------
+    encoded : dict
+      Has keys:
+        dtype: string of dtype
+        shape: int tuple of shape
+        base64: base64 encoded string of flat array
+        binary:  decode result coming from numpy.tostring
+
     Returns
     ----------
     array: numpy array
@@ -1048,12 +1136,15 @@ def is_instance_named(obj, name):
 
     Parameters
     ---------
-    obj: instance of a class
-    name: string
+    obj : instance
+      Some object of some class
+    name: str
+      The name of the class we want to check for
 
     Returns
     ---------
-    bool, whether the object is a member of the named class
+    is_instance : bool
+      Whether the object is a member of the named class
     """
     try:
         type_named(obj, name)
@@ -1080,7 +1171,8 @@ def type_bases(obj, depth=4):
 
 def type_named(obj, name):
     """
-    Similar to the type() builtin, but looks in class bases for named instance.
+    Similar to the type() builtin, but looks in class bases
+    for named instance.
 
     Parameters
     ----------
@@ -1101,37 +1193,51 @@ def type_named(obj, name):
     raise ValueError('Unable to extract class of name ' + name)
 
 
-def concatenate(a, b):
+def concatenate(a, b=None):
     """
-    Concatenate two meshes.
+    Concatenate two or more meshes.
 
     Parameters
     ----------
-    a: Trimesh object
-    b: Trimesh object
+    a: Trimesh object, or list of such
+    b: Trimesh object, or list of such
 
     Returns
     ----------
-    result: Trimesh object containing all faces of a and b
+    result: Trimesh object containing concatenated mesh
     """
-    # Extract the trimesh type to avoid a circular import,
+    if b is None:
+        b = []
+    # stack meshes into flat list
+    meshes = np.append(a, b)
+
+    # extract the trimesh type to avoid a circular import
     # and assert that both inputs are Trimesh objects
-    trimesh_type = type_named(a, 'Trimesh')
-    trimesh_type = type_named(b, 'Trimesh')
+    trimesh_type = type_named(meshes[0], 'Trimesh')
 
-    new_normals = np.vstack((a.face_normals, b.face_normals))
-    new_faces = np.vstack((a.faces, (b.faces + len(a.vertices))))
-    new_vertices = np.vstack((a.vertices, b.vertices))
-    new_visual = a.visual.concatenate(b.visual)
-    result = trimesh_type(vertices=new_vertices,
-                          faces=new_faces,
-                          face_normals=new_normals,
-                          visual=new_visual,
-                          process=False)
-    # result._cache.id_set()
-    # result.visual._cache.id_set()
+    # append faces and vertices of meshes
+    vertices, faces = append_faces(
+        [m.vertices.copy() for m in meshes],
+        [m.faces.copy() for m in meshes])
 
-    return result
+    # only save face normals if already calculated
+    face_normals = None
+    if all('face_normals' in m._cache for m in meshes):
+        face_normals = np.vstack([m.face_normals
+                                  for m in meshes])
+
+    # concatenate visuals
+    visual = meshes[0].visual.concatenate(
+        [m.visual for m in meshes[1:]])
+
+    # create the mesh object
+    mesh = trimesh_type(vertices=vertices,
+                        faces=faces,
+                        face_normals=face_normals,
+                        visual=visual,
+                        process=False)
+
+    return mesh
 
 
 def submesh(mesh,
@@ -1143,16 +1249,20 @@ def submesh(mesh,
 
     Parameters
     ----------
-    mesh: Trimesh object
-    faces_sequence: sequence of face indices from mesh
-    only_watertight: only return submeshes which are watertight.
-    append: return a single mesh which has the faces specified appended.
-            if this flag is set, only_watertight is ignored
+    mesh : Trimesh
+       Source mesh to take geometry from
+    faces_sequence : sequence (p,) int
+        Indexes of mesh.faces
+    only_watertight : bool
+        Only return submeshes which are watertight.
+    append : bool
+        Return a single mesh which has the faces appended,
+        if this flag is set, only_watertight is ignored
 
     Returns
     ---------
-    if append: Trimesh object
-    else:      list of Trimesh objects
+    if append : Trimesh object
+    else        list of Trimesh objects
     """
     # evaluate generators so we can escape early
     faces_sequence = list(faces_sequence)
@@ -1166,17 +1276,17 @@ def submesh(mesh,
         all_faces = np.array_equal(np.sort(faces_sequence),
                                    np.arange(len(faces_sequence)))
         if all_faces:
-            log.debug('entire mesh requested, returning copy of original')
+            log.debug('entire mesh requested, returning copy')
             return mesh.copy()
 
     # avoid nuking the cache on the original mesh
     original_faces = mesh.faces.view(np.ndarray)
     original_vertices = mesh.vertices.view(np.ndarray)
 
-    faces = collections.deque()
-    vertices = collections.deque()
-    normals = collections.deque()
-    visuals = collections.deque()
+    faces = []
+    vertices = []
+    normals = []
+    visuals = []
 
     # for reindexing faces
     mask = np.arange(len(original_vertices))
@@ -1200,25 +1310,33 @@ def submesh(mesh,
     # to avoid a circular import
     trimesh_type = type_named(mesh, 'Trimesh')
     if append:
-        visuals = np.array(visuals)
+        if all(hasattr(i, 'concatenate')
+               for i in visuals):
+            visuals = np.array(visuals)
+            visual = visuals[0].concatenate(visuals[1:])
+        else:
+            visual = None
+
         vertices, faces = append_faces(vertices, faces)
-        appended = trimesh_type(vertices=vertices,
-                                faces=faces,
-                                face_normals=np.vstack(normals),
-                                visual=visuals[0].concatenate(visuals[1:]),
-                                process=False)
+        appended = trimesh_type(
+            vertices=vertices,
+            faces=faces,
+            face_normals=np.vstack(normals),
+            visual=visual,
+            process=False)
         return appended
 
     # generate a list of Trimesh objects
-    result = [trimesh_type(vertices=v,
-                           faces=f,
-                           face_normals=n,
-                           visual=c,
-                           metadata=copy.deepcopy(mesh.metadata),
-                           process=False) for v, f, n, c in zip(vertices,
-                                                                faces,
-                                                                normals,
-                                                                visuals)]
+    result = [trimesh_type(
+        vertices=v,
+        faces=f,
+        face_normals=n,
+        visual=c,
+        metadata=copy.deepcopy(mesh.metadata),
+        process=False) for v, f, n, c in zip(vertices,
+                                             faces,
+                                             normals,
+                                             visuals)]
     result = np.array(result)
     if len(result) > 0 and only_watertight:
         # fill_holes will attempt a repair and returns the
@@ -1235,12 +1353,15 @@ def zero_pad(data, count, right=True):
     """
     Parameters
     --------
-    data: (n) length 1D array
-    count: int
+    data : (n,)
+      1D array
+    count : int
+      Minimum length of result array
 
     Returns
     --------
-    padded: (count) length 1D array if (n < count), otherwise length (n)
+    padded : (m,)
+      1D array where m >= count
     """
     if len(data) == 0:
         return np.zeros(count)
@@ -1255,18 +1376,21 @@ def zero_pad(data, count, right=True):
         return np.asanyarray(data)
 
 
-def jsonify(obj):
+def jsonify(obj, **kwargs):
     """
     A version of json.dumps that can handle numpy arrays
     by creating a custom encoder for numpy dtypes.
 
     Parameters
     --------------
-    obj: JSON- serializable blob
+    obj : JSON- serializable blob
+    **kwargs :
+        Passed to json.dumps
 
     Returns
     --------------
-    dumped: str, JSON dump of obj
+    dumped : str
+      JSON dump of obj
     """
     class NumpyEncoder(json.JSONEncoder):
 
@@ -1277,7 +1401,7 @@ def jsonify(obj):
                 return obj.tolist()
             return json.JSONEncoder.default(self, obj)
     # run the dumps using our encoder
-    dumped = json.dumps(obj, cls=NumpyEncoder)
+    dumped = json.dumps(obj, cls=NumpyEncoder, **kwargs)
     return dumped
 
 
@@ -1359,11 +1483,12 @@ def bounds_tree(bounds):
 
 def wrap_as_stream(item):
     """
-    Wrap a string or bytes object as a file object
+    Wrap a string or bytes object as a file object.
 
     Parameters
     ----------
-    item: str or bytes: item to be wrapped
+    item: str or bytes
+      Item to be wrapped
 
     Returns
     ---------
@@ -1375,22 +1500,22 @@ def wrap_as_stream(item):
         return StringIO(item)
     elif isinstance(item, bytes):
         return BytesIO(item)
-    raise ValueError('Not a wrappable item!')
+    raise ValueError('{} is not wrappable!'.format(type(item).__name__))
 
 
 def sigfig_round(values, sigfig=1):
     """
-    Round a single value to a specified number of signifigant figures.
+    Round a single value to a specified number of significant figures.
 
     Parameters
     ----------
     values: float, value to be rounded
-    sigfig: int, number of signifigant figures to reduce to
+    sigfig: int, number of significant figures to reduce to
 
 
     Returns
     ----------
-    rounded: values, but rounded to the specified number of signifigant figures
+    rounded: values, but rounded to the specified number of significant figures
 
 
     Examples
@@ -1413,12 +1538,12 @@ def sigfig_round(values, sigfig=1):
 def sigfig_int(values, sigfig):
     """
     Convert a set of floating point values into integers with a specified number
-    of signifigant figures and an exponent.
+    of significant figures and an exponent.
 
     Parameters
     ------------
     values: (n,) float or int, array of values
-    sigfig: (n,) int, number of signifigant figures to keep
+    sigfig: (n,) int, number of significant figures to keep
 
     Returns
     ------------
@@ -1500,7 +1625,10 @@ def compress(info):
     else:
         file_obj = StringIO()
 
-    with zipfile.ZipFile(file_obj, 'w') as zipper:
+    with zipfile.ZipFile(
+            file_obj,
+            mode='w',
+            compression=zipfile.ZIP_DEFLATED) as zipper:
         for name, data in info.items():
             if hasattr(data, 'read'):
                 # if we were passed a file object, read it
@@ -1522,7 +1650,7 @@ def split_extension(file_name, special=['tar.bz2', 'tar.gz']):
     Parameters
     ----------
     file_name: str, file name
-    special:   list of str, multipart exensions
+    special:   list of str, multipart extensions
                eg: ['tar.bz2', 'tar.gz']
 
     Returns
@@ -1543,7 +1671,7 @@ def triangle_strips_to_faces(strips):
     """
     Given a sequence of triangle strips, convert them to (n,3) faces.
 
-    Processes all strips at once using np.concatenate and is signifigantly
+    Processes all strips at once using np.concatenate and is significantly
     faster than loop- based methods.
 
     From the OpenGL programming guide describing a single triangle
@@ -1608,11 +1736,15 @@ def vstack_empty(tup):
     stacked: (n,d) array, with same number of columns as
               constituent arrays.
     """
+    # filter out empty arrays
     stackable = [i for i in tup if len(i) > 0]
+    # if we only have one array just return it
     if len(stackable) == 1:
-        return stackable[0]
+        return np.asanyarray(stackable[0])
+    # if we have nothing return an empty numpy array
     elif len(stackable) == 0:
         return np.array([])
+    # otherwise just use vstack as normal
     return np.vstack(stackable)
 
 
@@ -1658,7 +1790,7 @@ def unique_id(length=12, increment=0):
 
     Parameters
     ------------
-    length:    int, length of resuling identifier
+    length:    int, length of resulting identifier
     increment: int, number to add to header uint16
                     useful if calling this function repeatedly
                     in a tight loop executing faster than time
@@ -1684,23 +1816,123 @@ def unique_id(length=12, increment=0):
 
 def generate_basis(z):
     """
-    Generate an arbitrary basis (coordinate frame)
-    from the given z-axis.
+    Generate an arbitrary basis (also known as a coordinate frame)
+    from a given z-axis vector.
 
     Parameters
     ----------
-    z: (3,) float, a positive z-axis vector.
+    z: (3,) float
+      A vector along the positive z-axis
 
     Returns
     -------
-    x: (3,) float, the x axis
-    y: (3,) float, the y axis
-    z: (3,) float, the z axis
+    x : (3,) float
+      Vector along x axis
+    y : (3,) float
+      Vector along y axis
+    z : (3,) float
+      Vector along z axis
     """
-    z = z / np.linalg.norm(z)
+    # get a copy of input vector
+    z = np.array(z, dtype=np.float64, copy=True)
+    # must be a 3D vector
+    if z.shape != (3,):
+        raise ValueError('z must be (3,) float!')
+
+    # normalize vector in- place
+    z /= np.linalg.norm(z)
+    # X as arbitrary perpendicular vector
     x = np.array([-z[1], z[0], 0.0])
-    if np.linalg.norm(x) == 0.0:
+    # avoid degenerate case
+    if np.isclose(np.linalg.norm(x), 0.0):
+        # Z is already along Z [0, 0, 1]
+        # so a perpendicular X is just X
         x = np.array([1.0, 0.0, 0.0])
-    x = x / np.linalg.norm(x)
+    else:
+        # otherwise normalize X in- place
+        x /= np.linalg.norm(x)
+    # get perpendicular Y with cross product
     y = np.cross(z, x)
-    return x, y, z
+    # append result values into vector
+    result = np.array([x, y, z], dtype=np.float64)
+
+    return result
+
+
+def unique_bincount(values,
+                    minlength,
+                    return_inverse=True):
+    """
+    For arrays of integers find unique values using bin counting.
+    Roughly 20x faster for correct input than np.unique
+
+    Parameters
+    --------------
+    values : (n,) int
+      Values to find unique members of
+    minlength : int
+      Maximum value that will occur in values (values.max())
+    return_inverse : bool
+      If True, return an inverse such that unique[inverse] == values
+
+    Returns
+    ------------
+    unique : (m,) int
+      Unique values in original array
+    inverse : (n,) int
+      An array such that unique[inverse] == values
+      Only returned if return_inverse is True
+    """
+    values = np.asanyarray(values)
+    if len(values.shape) != 1 or values.dtype.kind != 'i':
+        raise ValueError('input must be 1D integers!')
+
+    try:
+        # count the number of occurrences of each value
+        counts = np.bincount(values, minlength=minlength)
+    except TypeError:
+        # casting failed on 32 bit windows
+        log.error('casting failed!', exc_info=True)
+        # fall back to numpy unique
+        return np.unique(values, return_inverse=return_inverse)
+
+    # which bins are occupied at all
+    unique_bin = counts > 0
+
+    # which values are unique
+    # indexes correspond to original values
+    unique = np.where(unique_bin)[0]
+
+    if return_inverse:
+        # find the inverse to reconstruct original
+        inverse = (np.cumsum(unique_bin) - 1)[values]
+        return unique, inverse
+
+    return unique
+
+
+def isclose(a, b, atol):
+    """
+    A replacement for np.isclose that does fewer checks
+    and validation and as a result is roughly 4x faster.
+
+    Note that this is used in tight loops, and as such
+    a and b MUST be np.ndarray, not list or "array-like"
+
+    Parameters
+    ----------
+    a : np.ndarray
+      To be compared
+    b : np.ndarray
+      To be compared
+    atol : float
+      Acceptable distance between `a` and `b` to be "close"
+
+    Returns
+    -----------
+    close : np.ndarray, bool
+      Per- element closeness
+    """
+    diff = a - b
+    close = np.logical_and(diff > -atol, diff < atol)
+    return close
